@echo off
rem *******************************************************************************
rem  Licensed Materials - Property of IBM
rem  (c) Copyright IBM Corporation 2014. All Rights Reserved.
rem  
rem  Note to U.S. Government Users Restricted Rights:  
rem  Use, duplication or disclosure restricted by GSA ADP Schedule 
rem  Contract with IBM Corp. 
rem  *******************************************************************************


REM **************************************************************
REM * Jazz application upgrade script for Jazz Reporting Service *
REM **************************************************************

rem flush errorlevel env. var in case it was unfortunately set in
rem the Windows session before running the script...
set ERRORLEVEL=

setlocal

rem Ensure the app_upgrade script was not called directly.
set INVALID_LAUNCH=false
if DEFINED SCRIPT_DIR goto :LBLLOCALMSG
set INVALID_LAUNCH=true
for /f "delims=" %%i in ('cd') do set START_DIR=%%i
set SCRIPT_DIR=%~dp0
set SCRIPT_DIR="%SCRIPT_DIR:~0,-1%"

:LBLLOCALMSG
rem Set locale messages
call %SCRIPT_DIR%\locale.bat
set MSG_FILE_NAME=MESSAGES%LANG_SUFFIX%.bat
set MSG_FILE_PATH=%SCRIPT_DIR%\messages\%MSG_FILE_NAME%

rem Exit in case of no admin privileges
net session >nul 2>&1
if NOT %ERRORLEVEL% == 0 (
	%MSG_FILE_PATH% NO_ADMIN_PRIVILEGES
)

rem Exit in case of invalid launch
if %INVALID_LAUNCH% == true (
	%MSG_FILE_PATH% INVALID_LAUNCH
)

rem To make the if statements below with an AND condition more readable
rem define the variable AND to be "if"
SET AND=if

REM *********************
REM * Arguments reading *
REM *********************

rem First argument is the kind of upgrading
rem Should be only -rs
if %1. == . (
	%MSG_FILE_PATH% MISSING_APP_TYPE
)
set FIRST_PARAM=%1
if %1 == -rs  goto :LBLSETPARAMS
%MSG_FILE_PATH% INVALID_APP_TYPE %1
exit /b 10

:LBLSETPARAMS
shift
set UPGRADED_APP=%FIRST_PARAM:~1%
rem Set parameters values from file
call %SCRIPT_DIR%\params\%UPGRADED_APP%_params%LANG_SUFFIX%.bat

rem Unset the default application context root to allow the user to set it
set APPLICATION_CONTEXT_ROOT=

rem Starting dir should be server dir
cd %SCRIPT_DIR%\..\..
for /f "delims=" %%i in ('cd') do set SERVER_DIR=%%i
if NOT "%START_DIR%" == "%SERVER_DIR%" (
	if "%1" == "-help" goto LBLHELP
	%MSG_FILE_PATH% INVALID_START_DIR %SCRIPT_NAME% "%SERVER_DIR%"
)

REM ***********************
REM * Parse the arguments *
REM ***********************

:LBLARGS
if %1_ == _ goto :LBLVERIFPARAM & REM Jump when all parameters are consumed
	if /I "%1" == "-oldApplicationHome" (
		if %2. == . %MSG_FILE_PATH% MISSING_PARAM_VALUE %1&exit /b 2
		if NOT EXIST %2 %MSG_FILE_PATH% INVALID_PARAM_DIR %1 %2&exit /b 2
		set OLD_APPLICATION_HOME=%2
		shift
		shift
		goto LBLARGS
	)
	if /I "%1" == "-applicationContextRoot" (
		if %2. == . %MSG_FILE_PATH% MISSING_PARAM_VALUE %1&exit /b 2
		set APPLICATION_CONTEXT_ROOT=%2
		shift
		shift
		goto LBLARGS
	)
	if /I "%1" == "-oldApplicationVersion" (
		if %2. == . %MSG_FILE_PATH% MISSING_PARAM_VALUE %1&exit /b 2
		set OLD_APPLICATION_VERSION=%2
		shift
		shift
		goto LBLARGS
	)
	if /I "%1" == "-newApplicationHome" (
		if %2. == . %MSG_FILE_PATH% MISSING_PARAM_VALUE %1&exit /b 2
		if NOT EXIST %2 %MSG_FILE_PATH% INVALID_PARAM_DIR %1 %2&exit /b 2
		set NEW_APPLICATION_HOME=%2
		shift
		shift
		goto LBLARGS
	)
	if /I "%1" == "-updateTomcatFiles" (
		if %2. == . %MSG_FILE_PATH% MISSING_PARAM_VALUE %1&exit /b 2
		if NOT %2_ == yes_ %AND% NOT %2_ == no_ %MSG_FILE_PATH% INVALID_PARAM_VALUE %1 %2&exit /b 2
		set UPDATE_TOMCAT_FILES=%2
		shift
		shift
		goto LBLARGS
	)
	if /I "%1" == "-oldApplicationTomcatHome" (
		if %2. == . %MSG_FILE_PATH% MISSING_PARAM_VALUE %1&exit /b 2
		set OLD_APP_TOMCAT_HOME=%2
		shift
		shift
		goto LBLARGS
	)
	if /I "%1" == "-newApplicationTomcatHome" (
		if %2. == . %MSG_FILE_PATH% MISSING_PARAM_VALUE %1&exit /b 2
		set NEW_APP_TOMCAT_HOME=%2
		shift
		shift
		goto LBLARGS
	)
	if /I "%1" == "-ignoreJTSVersionCheck" (
		set IGNORE_JTS_VERSION_CHECK=ignoreJTSVersionCheck
		shift
		goto LBLARGS
	)	
	if /I "%1" == "-newJTSHome" (
		if %2. == . %MSG_FILE_PATH% MISSING_PARAM_VALUE %1&exit /b 2
		if NOT EXIST %2 %MSG_FILE_PATH% INVALID_PARAM_DIR %1 %2&exit /b 2
		set NEW_JTS_HOME=%2
		shift
		shift
		goto LBLARGS
	)
	if /I "%1" == "-jtsContextRoot" (
		if %2. == . %MSG_FILE_PATH% MISSING_PARAM_VALUE %1&exit /b 2
		set JTS_CONTEXT_ROOT=%2
		shift
		shift
		goto LBLARGS
	)
	if /I "%1" == "-noPrompt" (
		set NO_PROMPT=noPrompt
		shift
		goto LBLARGS
	)
	if /I "%1" == "-noVerify" (
		set NO_VERIFY=false
		shift
		goto LBLARGS
	)
	if /I "%1" == "-overwrite" (
		set OVERWRITE=overwrite=yes
		shift
		goto LBLARGS
	)	

	if /I "%1" == "-noEditor" (
		set NO_EDITOR=true
		shift
		goto LBLARGS
	)

	if /I "%1" == "-noStepPrompt" (
		set NO_STEP_PROMPT=true
		shift
		goto LBLARGS
	)

	if /I "%1" == "-importReportsOnStartup" (
		set IMPORT_REPORTS_ON_STARTUP=importReportsOnStartup
		shift
		goto LBLARGS
	)	
	
	if /I "%1" == "-help" (
		goto LBLHELP
	)
	
REM For all other parameters, display a message with how to get help and then exit 2
%MSG_FILE_PATH% INVALID_PARAM %1
exit /b 2


REM *************************
REM * Verify the parameters *
REM *************************
rem
rem  -oldApplicationHome: 		mandatory, no default
rem  -applicationContextRoot:	optional, default = rs
rem  -oldApplicationVersion:	optional, no default
rem  -newApplicationHome: 		mandatory, default = <scriptDirectory>/conf
rem  -updateTomcatFiles:		optional, default = yes
rem  -oldApplicationTomcatHome:	optional, default=tomcat
rem  -newApplicationTomcatHome:	optional, default=tomcat
rem
rem For possible verification of the JTS version against the upgraded app version 
rem the following parameters are needed
rem  -ignoreJTSVersionCheck:	optional, default is check app version against JTS version
rem  -newJTSHome: 				optional, default = value of -newApplicationHome
rem  -jtsContextRoot: 			optional, default = jts

:LBLVERIFPARAM


REM Determine the application context root and the previous application version
REM ---------------------------------------------------------------------------
set REAL_CONTEXT_ROOT=%SCRIPT_NAME:_upgrade.bat=%
if NOT "%REAL_CONTEXT_ROOT%" == "" %AND% "%APPLICATION_CONTEXT_ROOT%" == "" (
	set APPLICATION_CONTEXT_ROOT=%REAL_CONTEXT_ROOT%
)
if DEFINED OLD_APPLICATION_VERSION (
	set OLD_APPLICATION_VERSION_ARG=oldApplicationVersion=%OLD_APPLICATION_VERSION%
)


REM Verify that oldApplicationHome was specified.
REM --------------------------------------------------------------------
if NOT DEFINED OLD_APPLICATION_HOME (
	%MSG_FILE_PATH% MISSING_HOME_PARAM -oldApplicationHome
	exit /b 3
)
rem ensure variables for directories are in quotes (to handle spaces in names)
set TMP_DIR=%OLD_APPLICATION_HOME:"=%
set OLD_APPLICATION_HOME="%TMP_DIR%"
if NOT EXIST %OLD_APPLICATION_HOME% (
	%MSG_FILE_PATH% INVALID_PARAM_DIR -oldApplicationHome %OLD_APPLICATION_HOME%
	exit /b 3
)
if NOT "%TMP_DIR:~1,2%" == ":\" (
	%MSG_FILE_PATH% RELATIVE_PARAM_DIR -oldApplicationHome %OLD_APPLICATION_HOME%
	exit /b 3
)

cd /d %OLD_APPLICATION_HOME%\..
for /f "delims=" %%i in ('cd') do set OLD_APP_PARENT_DIR="%%i"


REM Verify old application home has an app.properties file
REM -------------------------------------------------------------
set OLD_APP_TSP_FILE=%OLD_APPLICATION_HOME:"=%\%APPLICATION_CONTEXT_ROOT%\app.properties
set OLD_APP_TSP_FILE="%OLD_APP_TSP_FILE%"
if NOT EXIST %OLD_APP_TSP_FILE% (
	%MSG_FILE_PATH% TSP_NOT_FOUND_OLD_APP %OLD_APP_TSP_FILE% %OLD_APPLICATION_HOME% %APPLICATION_CONTEXT_ROOT%
	exit /b 3
)


REM Verify new application home has an app.properties file
REM -------------------------------------------------------------
if NOT DEFINED NEW_APPLICATION_HOME (
	%MSG_FILE_PATH% MISSING_HOME_PARAM -newApplicationHome
	exit /b 3
)
set TMP_DIR=%NEW_APPLICATION_HOME:"=%
if NOT "%TMP_DIR%" == "." (
	if "%TMP_DIR:~1,2%" == ":\" (
		set NEW_APPLICATION_HOME="%TMP_DIR%"
	) else (
		set NEW_APPLICATION_HOME="%START_DIR%\%TMP_DIR%"
	)
) else (
	set NEW_APPLICATION_HOME="%START_DIR%"
)
if NOT EXIST %NEW_APPLICATION_HOME% (
	%MSG_FILE_PATH% DIR_NOT_FOUND %NEW_APPLICATION_HOME%
	exit /b 3
)
set NEW_APP_TSP_FILE=%NEW_APPLICATION_HOME:"=%\%APPLICATION_CONTEXT_ROOT%\app.properties
set NEW_APP_TSP_FILE="%NEW_APP_TSP_FILE%"
if NOT EXIST %NEW_APP_TSP_FILE% (
	%MSG_FILE_PATH% TSP_NOT_FOUND_NEW_APP %NEW_APP_TSP_FILE% %NEW_APPLICATION_HOME% %APPLICATION_CONTEXT_ROOT% %PREVIOUS_VERSION%
	exit /b 3
)

REM Verify new application home has a com.ibm.team.integration.reporting.upgrade.jar
REM ------------------------------------------------------------------------
cd /d %NEW_APPLICATION_HOME%\..
for /f "delims=" %%i in ('cd') do set NEW_APP_PARENT_DIR="%%i"
set UPGRADE_JAR=upgrade\%APPLICATION_CONTEXT_ROOT%\com.ibm.team.integration.reporting.upgrade.jar
if NOT EXIST %NEW_APP_PARENT_DIR%\%UPGRADE_JAR% (
	%MSG_FILE_PATH% CMD_NOT_FOUND_NEW_APP %NEW_APP_PARENT_DIR%\%UPGRADE_JAR% %NEW_APPLICATION_HOME% %APPLICATION_CONTEXT_ROOT% %PREVIOUS_VERSION%
	exit /b 3
)
set CLASSPATH=%UPGRADE_JAR%;upgrade\%APPLICATION_CONTEXT_ROOT%\*
set JAVA=jre\bin\java
set NEW_APPLICATION_CMD=%JAVA% com.ibm.team.integration.reporting.upgrade.JRSTools

REM Verify Tomcat parameters: updateTomcatFiles, oldApplicationTomcatHome and newApplicationTomcatHome
REM --------------------------------------------------------------------------------------------------
if DEFINED UPDATE_TOMCAT_FILES (
	set UPDATE_TOMCAT_FILES_ARG=updateTomcatFiles=%UPDATE_TOMCAT_FILES%
)

rem Old Application Tomcat home
if NOT DEFINED OLD_APP_TOMCAT_HOME goto :LBLENDVERIF1
set TMP_DIR=%OLD_APP_TOMCAT_HOME:"=%
set OLD_APP_TOMCAT_HOME="%TMP_DIR%"
if NOT "%TMP_DIR:~1,2%" == ":\" (
	%MSG_FILE_PATH% RELATIVE_PARAM_DIR -oldApplicationTomcatHome %OLD_APP_TOMCAT_HOME%
	exit /b 3
)
if NOT EXIST %OLD_APP_TOMCAT_HOME% (
	if NOT EXIST %OLD_APP_PARENT_DIR%\%OLD_APP_TOMCAT_HOME% (
		if %OLD_APP_TOMCAT_HOME::=% == %OLD_APP_TOMCAT_HOME% (
			%MSG_FILE_PATH% INVALID_PARAM_DIR -oldApplicationTomcatHome %OLD_APP_PARENT_DIR%%OLD_APP_TOMCAT_HOME%
		) else (
			%MSG_FILE_PATH% INVALID_PARAM_DIR -oldApplicationTomcatHome %OLD_APP_TOMCAT_HOME%
		)
		exit /b 3
	)
)
set OLD_APP_TOMCAT_HOME_ARG=oldTomcatHome=%OLD_APP_TOMCAT_HOME%

:LBLENDVERIF1

rem New Application Tomcat home
if NOT DEFINED NEW_APP_TOMCAT_HOME goto :LBLENDVERIF2
set TMP_DIR=%NEW_APP_TOMCAT_HOME:"=%
if NOT "%TMP_DIR%" == "." (
	if "%TMP_DIR:~1,2%" == ":\" (
		set NEW_APP_TOMCAT_HOME="%TMP_DIR%"
	) else (
		set NEW_APP_TOMCAT_HOME="%START_DIR%\%TMP_DIR%"
	)
) else (
	set NEW_APP_TOMCAT_HOME="%START_DIR%"
)
if NOT EXIST %NEW_APP_TOMCAT_HOME% (
	if NOT EXIST %NEW_APP_PARENT_DIR%\%NEW_APP_TOMCAT_HOME% (
		if %NEW_APP_TOMCAT_HOME::=% == %NEW_APP_TOMCAT_HOME% (
			%MSG_FILE_PATH% INVALID_PARAM_DIR -newApplicationTomcatHome %NEW_APP_PARENT_DIR%\%NEW_APP_TOMCAT_HOME%
		) else (
			%MSG_FILE_PATH% INVALID_PARAM_DIR -newApplicationTomcatHome %NEW_APP_TOMCAT_HOME%
		)
		exit /b 3
	)
)
set NEW_APP_TOMCAT_HOME_ARG=newApplicationTomcatHome=%NEW_APP_TOMCAT_HOME%

:LBLENDVERIF2


REM Verify the parameters needed for JTS and app version checks
REM -----------------------------------------------------------
if NOT DEFINED NEW_JTS_HOME (
	set NEW_JTS_HOME=%NEW_APPLICATION_HOME%
)
set TMP_DIR=%NEW_JTS_HOME:"=%
if NOT "%TMP_DIR%" == "." (
	if "%TMP_DIR:~1,2%" == ":\" (
		set NEW_JTS_HOME="%TMP_DIR%"
	) else (
		set NEW_JTS_HOME="%START_DIR%\%TMP_DIR%"
	)
) else (
	set NEW_JTS_HOME="%START_DIR%"
)
if NOT EXIST %NEW_JTS_HOME% (
	%MSG_FILE_PATH% INVALID_PARAM_DIR -newJTSHome %NEW_JTS_HOME%
	exit /b 3
)
if NOT DEFINED JTS_CONTEXT_ROOT (
	set JTS_CONTEXT_ROOT=jts
)

REM **************
REM * Start exec *
REM **************

:LBLSTARTEXEC
call %MSG_FILE_PATH% BANNER_%UPGRADED_APP%

REM *********************
REM * First step switch *
REM *********************

cd %START_DIR%
set STEP_ERROR=0
set /A INITIAL_STEP=0

rem Compute the maximum steps to execute
if %UPGRADED_APP% == rs (
	set /A MAX_STEPS=1
)
set /A LAST_STEP=-1
set /A LAST_RETURN=0
set VERIF_CMD=%SCRIPT_DIR%\app_verify.bat
set COPY_DATABASE_CMD=%SCRIPT_DIR%\copy_database.bat
set LOG_FILE=%SCRIPT_DIR%\.%UPGRADED_APP%_upgrade.log
if EXIST %LOG_FILE% (
	for /f "tokens=1,2,3" %%i in ('type %LOG_FILE%') do (
		set /A LAST_STEP=%%i
		set /A LAST_RETURN=%%j
		if NOT "%%k" == "" set LAST_STEP_INFO=%%k
	)
)
set /A CURRENT_STEP=%INITIAL_STEP%
goto :LBLENDVERIF

rem debug: echo MAX_STEPS=%MAX_STEPS%

:LBLFIRSTSTEP
REM Script was already executed
if %LAST_RETURN% EQU 0 (
	REM Previous execution finished without any error
	set /A CURRENT_STEP = LAST_STEP + 1
	if %LAST_STEP% GEQ %MAX_STEPS% goto :LBLFIRSTSTEP0
	goto :LBLFIRSTSTEP1
)

REM Previous run finished with an error, restart from failing step
set /A CURRENT_STEP = LAST_STEP
if %LAST_STEP% EQU %INITIAL_STEP% (
	call %MSG_FILE_PATH% INITIAL_STEP_FAILED
	goto LBLSTEP%CURRENT_STEP%
)
call %MSG_FILE_PATH% LAST_STEP_FAILED %LAST_STEP%
set /A STEP_NUM_MAX=%LAST_STEP%
goto :LBLFIRSTSTEPSUB

:LBLFIRSTSTEP0
call %MSG_FILE_PATH% ALREADY_RUN
set /A STEP_NUM_MAX=%MAX_STEPS%
goto :LBLFIRSTSTEPSUB

:LBLFIRSTSTEP1
call %MSG_FILE_PATH% LAST_STEP_PASSED %LAST_STEP% %CURRENT_STEP%
set /A STEP_NUM_MAX=%CURRENT_STEP%
goto :LBLFIRSTSTEPSUB

:LBLFIRSTSTEPSUB
set ANSWER=
call %MSG_FILE_PATH% ENTER_NEXT_STEP %INITIAL_STEP% %STEP_NUM_MAX%
set /P ANSWER=
if "%ANSWER%" == "" goto LBLEND
set /A STEP_NUMBER=%ANSWER%
if NOT %ANSWER% == 0 if %STEP_NUMBER% EQU 0 (
	call %MSG_FILE_PATH% INTEGER_STEP_NUMBER %ANSWER%
	goto :LBLFIRSTSTEPSUB
)
if %STEP_NUMBER% LSS %INITIAL_STEP% (
	call %MSG_FILE_PATH% INVALID_STEP_NUMBER %ANSWER%
	goto :LBLFIRSTSTEPSUB
)
if %STEP_NUMBER% GTR %MAX_STEPS% (
	call %MSG_FILE_PATH% MAX_STEP_NUMBER %MAX_STEPS% %STEP_NUMBER%
	goto :LBLFIRSTSTEPSUB
)
if %STEP_NUMBER% GTR %STEP_NUM_MAX% (
	call %MSG_FILE_PATH% NUM_MAX_STEP_NUMBER %STEP_NUM_MAX% %STEP_NUMBER%
	goto :LBLFIRSTSTEPSUB
)
set /A CURRENT_STEP=%STEP_NUMBER%
if %CURRENT_STEP% EQU %INITIAL_STEP% goto LBLENDVERIF


REM **********************
REM * Verify Public URIs *
REM **********************

if "%NO_VERIFY%" == "false" goto LBLENDVERIF
if NOT EXIST %VERIF_CMD% goto LBLENDVERIF
call %VERIF_CMD%
if NOT %ERRORLEVEL% == 0 goto LBLEND

:LBLENDVERIF
goto LBLSTEP%CURRENT_STEP%

REM **************
REM * Steps loop *
REM **************

REM =======================================
REM = Step 0 - Update Configuration Files =
REM =======================================

:LBLSTEP0
set CURRENT_STEP_MSG=STEP_0
set STEP_DIR=%NEW_APP_PARENT_DIR%
set CMD_ARGUMENTS=oldApplicationHome=%OLD_APPLICATION_HOME% applicationContextRoot=%APPLICATION_CONTEXT_ROOT% %OLD_APPLICATION_VERSION_ARG% newApplicationHome=%NEW_APPLICATION_HOME% %UPDATE_TOMCAT_FILES_ARG% %OLD_APP_TOMCAT_HOME_ARG% %NEW_APP_TOMCAT_HOME_ARG% %IGNORE_JTS_VERSION_CHECK% newJTSHome=%NEW_JTS_HOME% jtsContextRoot=%JTS_CONTEXT_ROOT% %IMPORT_REPORTS_ON_STARTUP% 
set STEP_CMD=%NEW_APPLICATION_CMD% -migration_%UPGRADED_APP%_updateConfigurationFiles %CMD_ARGUMENTS%
goto LBLRUNSTEP

REM ==========================
REM = Step 1 - Copy Database =
REM ==========================

:LBLSTEP1
set CURRENT_STEP_MSG=STEP_1
REM Copy the JRS application Jena database directory (server/conf/$app/db) only if it exists 
REM in the old application home and has not been copied yet to the new application home.
set STEP_DIR=%NEW_APP_PARENT_DIR%
set CURRENT_STEP_MSG=STEP_1_COPYDB
if NOT EXIST %COPY_DATABASE_CMD% goto LBLEND_MIGRATION_FAILED
set STEP_CMD=%COPY_DATABASE_CMD% %OLD_APPLICATION_HOME% %APPLICATION_CONTEXT_ROOT% %NEW_APPLICATION_HOME% %APPLICATION_CONTEXT_ROOT%
goto LBLRUNSTEP

:LBLSTEP2
rem Script does not support JTS migration
goto LBLEND_MIGRATION_SUCCESS


REM *********************
REM * Step run function *
REM *********************

:LBLRUNSTEP
cd /d %STEP_DIR%
echo.
echo.
call %MSG_FILE_PATH% %CURRENT_STEP_MSG% %CURRENT_STEP%
call %MSG_FILE_PATH% STEP_DIR_CMD %STEP_DIR%
set TAB=    
echo %TAB%%STEP_CMD%
echo.
set RUN_STEP=
if NOT "%NO_STEP_PROMPT%" == "true" (
	call %MSG_FILE_PATH% ENTER_STEP_ACTION
	set /P RUN_STEP=
	echo.
) else (
	set RUN_STEP=E
)
if /I "%RUN_STEP%" == "E" goto :LBLRUNSTEP1
if /I "%RUN_STEP%" == "S" (
	if %CURRENT_STEP% EQU 0 goto :LBLRUNVERIF2
	goto :LBLRUNSTEP2
)
if /I "%RUN_STEP%" == "" goto :LBLEND
call %MSG_FILE_PATH% INVALID_STEP_ACTION %RUN_STEP%
goto LBLRUNSTEP

:LBLRUNSTEP1
call %STEP_CMD%
set STEP_ERROR=%ERRORLEVEL%
if NOT %STEP_ERROR% EQU 0 goto :LBLRUNSTEP2
if %CURRENT_STEP% NEQ 0 goto :LBLRUNSTEP2

REM Finish application app.properties setup manually on step 0
:LBLRUNVERIF1
if "%NO_EDITOR%" == "true" goto :LBLRUNVERIF2
call %MSG_FILE_PATH% EDIT_APP_PROPERTIES %UPGRADED_APP%
call %MSG_FILE_PATH% PAUSE
set PAUSE=
set /P PAUSE=
if "%PAUSE%" == "" goto :LBLEND
if /I NOT "%PAUSE%" == "C" (
	call %MSG_FILE_PATH% INVALID_PAUSE_ACTION %PAUSE%
	goto :LBLRUNVERIF1
) 
notepad %NEW_APP_TSP_FILE%

:LBLRUNVERIF2
REM Verify URIs in app.properties files
if "%NO_VERIFY%" == "false" goto :LBLRUNSTEP2
if NOT EXIST %VERIF_CMD% goto :LBLRUNSTEP2
call %VERIF_CMD%
if NOT %ERRORLEVEL% == 0 goto LBLEND_MIGRATION_FAILED

:LBLRUNSTEP2
attrib -h %LOG_FILE% > nul
if %CURRENT_STEP% LSS %MAX_STEPS% (
		echo %CURRENT_STEP% %STEP_ERROR% > %LOG_FILE%
) else (
	echo %CURRENT_STEP% %STEP_ERROR% > %LOG_FILE%
)
attrib +h %LOG_FILE%
if NOT %STEP_ERROR% EQU 0 goto LBLEND_MIGRATION_FAILED

:LBLRUNSTEP3
set /A CURRENT_STEP = CURRENT_STEP + 1
if %CURRENT_STEP% GTR %MAX_STEPS% (
	goto LBLEND_MIGRATION_SUCCESS
)
goto LBLSTEP%CURRENT_STEP%


REM *****************
REM * Help messages *
REM *****************

:LBLHELP
type %SCRIPT_DIR%\doc\%UPGRADED_APP%_help%LANG_SUFFIX%.txt
set STEP_ERROR=1
goto LBLEND

REM **********************
REM * Script end message *
REM **********************

:LBLEND_MIGRATION_FAILED
call %MSG_FILE_PATH% OVERALL_MIGRATION_FAILED %UPGRADED_APP% %CURRENT_STEP%
goto LBLEND

:LBLEND_MIGRATION_SUCCESS
if %UPGRADED_APP% == rs (
	call %MSG_FILE_PATH% OVERALL_MIGRATION_SUCCESS_RS %UPGRADED_APP%
)
goto LBLEND

REM **************
REM * Script end *
REM **************

:LBLEND
endlocal & exit /b %STEP_ERROR%
