/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2015. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("com.ibm.rdm.web.grid.Grid");

dojo.require("dijit._Templated");
dojo.require("dijit._Widget");

dojo.require("jazz.ui.toolbar.Button");

dojo.require("com.ibm.rdm.web.common.ErrorHandler");
dojo.require("com.ibm.rdm.web.grid.Module");
dojo.require("com.ibm.rdm.web.grid.VirtualGrid");
dojo.require("com.ibm.rdm.web.grid.ViewHeader");
dojo.require("com.ibm.rdm.web.grid.GridModel");
dojo.require("com.ibm.rdm.web.grid.GroupManager");
dojo.require("com.ibm.rdm.web.grid.PagingWidget");
dojo.require("com.ibm.rdm.web.notifications.ChangeNotifier");
dojo.require("com.ibm.rdm.web.notifications.EventTopics");


var ErrorHandler = com.ibm.rdm.web.common.ErrorHandler;
var ChangeNotifier = com.ibm.rdm.web.notifications.ChangeNotifier;

var MESSAGE_ROW = "_mr";
var message_count = 0;

// This is set to either "ctrlKey" (windows) or "metaKey" (Mac) to indicate whether to look
// for meta or ctrl as the modifier for copy/paste keyboard operations.
// Regardless of which browser is used on the Mac, we make use of the "Command"/"Option"
// variants rather than the "Control"/"Alt" variants.
var COPYKEY_MODIFIER = (
	dojo.isMac ? "metaKey" : "ctrlKey"
);

var ArtifactState = {
	COLLAPSED: "collapsed",
	EXPANDED: "expanded",
	BOTH: "both"
};

dojo.declare(
		"com.ibm.rdm.web.grid.Grid",
		[dijit._Widget, dijit._Templated],
{
	templatePath: dojo.moduleUrl("com.ibm.rdm.web", "grid/templates/Grid.html"),
	widgetsInTemplate: true,

	_addToNewArtifacts: null,
	_changeNotifierListeners: null,
	_lastHandledEvent: null,
	
	/* If this is present, it is used to defer display actions until the deferred is calledback.
	   It is a function, which may return null - in which case actions are synchronous, or it
	   can return a deferred - in which case, the action is deferred till callback.*/
	preDisplayCallbackFunction: null,
	
	// Added by ebcraig -
	// used in case of lightweight grid such as in reporting wizard
	// to prevent normal keypress/click handlers from interfering with function
	// as intended
	skipModuleHandlerHookup: false,
	
	// May be set on construction.
	// If false, tabbing into and out of the grid is not managed, but tabbing follows
	// the normal browser behaviour, moving between input and other elements that have
	// tabIndex set on them.
	// If true (default), the grid catches tabs into it and clicks on cells, so that
	// the grid can be navigated with arrow keys, rows placed in ActionableMode etc.
	// In this case, all cell content must not be focusable by default, and methods must
	// be provided in the gridmodel column definitions to handle editing.  
	manageFocusNavigation: true,
	
	// May be set on construction.
	// On an instance with manageFocusNavigation==false, this flag may be set to
	// allow the rows to be tab focus targets, and thereby allowing keyboard selection
	// of rows.
	// Is only ever used when manageFocusNavigation is false.  When manageFocusNavigation
	// is true, rows are always selectable.
	manageRowSelection: false,
	
	// doTrackFocusForCells: Boolean
	//		Optionally passed into constructor.
	//		If true, rows will have the class com.ibm.rdm.web.grid.Module.ClassNames.ROW_FOCUS
	//		as long as a cell in that row has focus.
	doTrackFocusForCells: false,

	// fixedHeight: Number
	//		The height that this grid should use, as a number, representing the height in pixels.
	fixedHeight: null,

	module: null,
	header: null,
	
	message: null,
	currentArgs: null,
	
	restoreAfterRefresh: false,
	
	// The top and bottom visible row item in the viewport.
	// Used to restore the scroll position after refresh.
	lastTopVisibleItem: null, 
	lastBottomVisibleItem: null,
	
	// isRefresh: Boolean
	//		If true, it means the grid is doing refresh and we need to re-apply the module levels to display
	//		which will trigger onChange event.
	isRefresh: false,

	// showWithFindButton: Boolean
	//		If passed as true, you must override the onFind method.
	showWithFindButton: false,

	// accountForPagingWidgetWhenResettingHeight: Boolean
	//		If passed as true, when the grid is asked to change its dimensions
	//		it will consider the height of the PagingWidget (displayed above
	//		the header) when deciding how much height it has to display all
	//		the other elements that it manages (header, content area and footer).
	//		You may wish to pass this field as false if you're using a CSS
	//		approach to making sure that the ViewHeader remains at a "fixed Y
	//		position", rather than getting "pushed down" whenever the pager
	//		appears or disappears (depending on whether it's got anything useful
	//		to show).
	accountForPagingWidgetWhenResettingHeight: true,

	// accountForFooterWhenResettingHeight: Boolean
	//		If passed as false, calls to 'resize' and 'setHeight' will not
	//		automatically assume there's a 20 pixel high footer that needs
	//		to be accounted for.
	accountForFooterWhenResettingHeight: true,

/////////////////////////////////////////////////////////////////////////////////
//// Creation and Initialization

	postCreate: function() {
		// It is only valid to set manageRowSelection if we are not managing focus, but are using browser default focus behaviour,
		// so make sure the flags have been set to a valid state on construction.
		if (this.manageFocusSelection) {
			this.manageRowSelection = false;
		}

		this._changeNotifierListeners = [];

		if (this.virtualScrolling) {
			this.module = new com.ibm.rdm.web.grid.VirtualGrid({
				pageModel: this.pageModel,
				scrollParent: this.moduleContentPane,
				keyPressHandler: dojo.hitch(this, this._keyPressHandler),
				keyDownHandler: dojo.hitch(this, this._keyDownHandler),
				preDisplayCallbackFunction: this.preDisplayCallbackFunction,
				onNodeFocused: dojo.hitch(this, this.onNodeFocused)
			}, this.moduleDiv);
		} else {
			this.module = new com.ibm.rdm.web.grid.Module({
				keyPressHandler: dojo.hitch(this, this._keyPressHandler),
				keyDownHandler: dojo.hitch(this, this._keyDownHandler),
				skipHandlerHookup: this.skipModuleHandlerHookup,
				manageFocusNavigation: this.manageFocusNavigation,
				manageRowSelection: this.manageRowSelection,
				doTrackFocusForCells: this.doTrackFocusForCells,
				onNodeFocused: dojo.hitch(this, this.onNodeFocused)
			}, this.moduleDiv);
		}
		
		if (this.enableDrag) {
			this.module.enableDrag = true;
			this.module.setDragDropArgs(this.dragDropArgs);
			dojo.addClass(this.domNode, "dragEnabled");
		}
		
		this.header = new com.ibm.rdm.web.grid.ViewHeader(
			{
				showWithFindButton: this.showWithFindButton,
				onFind: this.onFind
			},
			this.viewHeaderDiv
		);

		this.connect(this.header, "onColumnResize", function (where) {
			
			this._position = this._position || dojo.position(this.domNode);

			dojo.style(this.resizeIndicator, {
				"left": (where - (this._position.x))+"px",
				"display": "block"
			});
			
			
			if (!dojo.isIE) {
				// Chrome has the default behaviour of changing the cursor
				// to text when you press the mouse down.
				if (!this._resizingColumn) {
					dojo.style(this.domNode, "webkitUserSelect", "none");
					dojo.style(this.domNode, "userSelect", "none");
					dojo.setSelectable(this.domNode, false);
					this._resizingColumn = true;
				}
			}
		});
		

		this.header.addColumnListener(this.module);
		
		this.pagingWidget = new com.ibm.rdm.web.grid.PagingWidget(
			{
				onHide: this.onHidePagingWidget,
				onShow: this.onShowPagingWidget
			},
			this.pagingWidgetDiv
		);
		this.connect(this.pagingWidget, "hide", function() {
			this._updateCssClassAttributesToHidePaging();
			if (this.currentArgs) {
				this.setHeight(this.currentArgs.h);
			}
		});
		this.connect(this.pagingWidget, "show", function() {
			this._updateCssClassAttributesToShowPaging();
			if (this.currentArgs) {
				this.setHeight(this.currentArgs.h);
			}
		});
		
		dojo.style(this.resizeIndicator, "top",  this.pagingWidget.domNode.offsetHeight+"px");
		
		this.gridmodel = new com.ibm.rdm.web.grid.GridModel({_underlyingGrid: this});
		
		this.gridmodel.postCreate();
		// this is purely so we can make the column wide enough, there does not seem to
		// be a pleasant way to do this...
		if (this.enableDrag) {
			this.gridmodel.enableDrag = true;
		}
		this.module.setGridmodel(this.gridmodel);
		this.header.setGridmodel(this.gridmodel);
				
		//Initialise the grid with basic types
		var message_col = {
			valueKey: "message", isVisible: true, width: 400
		};
		
		//Define the row type
		this.message = "";
		var self = this;
		this.gridmodel.addRowType(MESSAGE_ROW, {isSelectable: false, hasSelectionBox: false, hasMenu: false, isPrimary: false, showFullTitleOnHover: true, 
											rowDataFunction: function() {return {message: self.message};}});
		
		
		//Add the single message column
		this.gridmodel.addColumn(MESSAGE_ROW, "_mr-col", message_col);
		
		if (this.fixedHeight) {
			this.setHeight(this.fixedHeight);
		}
	},
	
	onNodeFocused: function(/*DOMNode*/node) {
		// summary:
		//		User defined function to handle the case of a new node becoming
		//		selected in the grid.
		// tags:
		//		callback
	},

	onFind: function(/*DOMNode*/findNode) {
		// summary:
		//		User defined function to handle the case of the user clicking
		//		on the Find button in this widget (note: the Find button will
		//		only be present if you've passed showWithFindButton as true
		//		when constructing this widget).
		// tags:
		//		callback
	},

	onHidePagingWidget: function() {
		// summary:
		//		User defined function to handle the case of the paging widget
		//		getting hidden (i.e. 'display' set to 'none').
		// tags:
		//		callback
	},

	onShowPagingWidget: function() {
		// summary:
		//		User defined function to handle the case of the paging widget
		//		getting shown (i.e. 'display' set to '').
		// tags:
		//		callback
	},

	startup: function() {

		this.module.startup();
		this.header.startup();

		this._connections = [];
		//Connect everything up
		this._connections.push(dojo.connect(this.header, "selectAll", this.module, "selectAll"));
		this._connections.push(dojo.connect(this.module, "setHeaderCheckbox", this.header, "setHeaderCheckbox"));
		this._connections.push(dojo.connect(this.header, "sortBy", this.module, "sortBy"));
		this._connections.push(dojo.connect(this.header, "hideCol", this, "removeCol"));
		this._connections.push(dojo.connect(this.header, "showCol", this, "addCol"));
		this._connections.push(dojo.connect(this.header, "reapplyGridDialogData", this, "_reapplyGridDialogData"));
		this._connections.push(dojo.connect(this.gridmodel, "_notifyRowDeletes", this, "notifyRowDeletes"));
		this._connections.push(dojo.connect(this.module, "onSaveKey", this, "_onSaveKey"));
		this._connections.push(dojo.connect(this.module, "onEscKey", this, "_onEscKey"));
		this._connections.push(dojo.connect(this.module, "onCtrlDKey", this, "_onCtrlDKey"));


		//Now lets start stuff up

		//Configure the module.
		//this line causes empty views in cases. don't know why so commenting out for now
		//this.module.clear();

		
		// Make sure that the header and the content
		// are aligned on column resize.
		var that = this;
		this.header.addColumnListener({
			setColumnWidth : function () {
				that.scrollHeaderToMatchContent();
				// hide the vertical resize bar
				dojo.style(that.resizeIndicator, "display", "none");
				if (!dojo.isIE) {
					dojo.style(that.domNode, "webkitUserSelect", "text");
					dojo.setSelectable(that.domNode, true);
					dojo.style(that.domNode, "userSelect", "text");
					that._resizingColumn = false;
				}
			}
		});
		
		if (this.virtualScrolling) {		
			this.connect(this.moduleContentPane, "onscroll", function (){
				this.module.onBodyScroll();
			});		

			// if we have virtual scrolling configured, listen for the
			// get page message.
			this.connect(this.module, "onGetPage", function (pageNum) {
				// TODO: supply list of URIs here and use these instead
				// of pageNum in RSGV and DashboardTabPage
				this.pagingWidget._gotoPageFunction(pageNum);
			});
			this.connect(this.module, "requestOrReconcileArtifacts", this.requestOrReconcileArtifacts);
			this.connect(this.module, "requestArtifacts", this.requestArtifacts);
			this.connect(this.module, "releaseArtifacts", this.releaseArtifacts);
		}
		
		this.connect(this.moduleContentPane, "onscroll", function () {
			that.scrollHeaderToMatchContent();
		});
		
		this._started = true;
	},
	
	requestOrReconcileArtifacts: function(resources, uris, callback){
		// summary:
		//		Invoked by virtual grid when it may or may not need artifacts
		//		uses artifacts in memory when possible (code lies in
		//		resultSetGridView).
	},

	requestArtifacts: function (uris, callback) {
		// summary:
		//		Invoked by the virtual grid when it needs new artifacts
		//		to be loaded.
	},
	
	refreshArtifacts: function(resourceURIs, includeLockInfo) {
		// summary:
		//		Invoked to refresh & redraw artifact displayed in the grid
		//		as well as trigger appropriate updates in data cache, etc.
	},
	
	releaseArtifacts: function (keys) {
		// summary:
		//		Invoked by the virtual grid when it no longer
		//		requires "keys" to be kept in the DataCache.
	},
	
	setColumnReorderNotification: function(reorderNotification) {
		// summary:
		//		If set, allows the setter to receive a call back when the
		//		columns are reordered reorderNotification.
		this._reorderNotification = reorderNotification;
	},
	
	resize: function(args) {
		//Get the header to resize the col widths
		if (!args) {
			return;
		}
		
		delete this._position;
		
		this.header.resize(args.w);

		if (!this.fixedHeight) {
			//Set the height
			this.setHeight(args.h);
		} else {
			args.h = this.fixedHeight;
		}
		
		this.currentArgs = args;
		this.module.updateLinkBoxPosition();
	},
	
	setHeight: function(height) {
		// summary:
		//		Sets the height of the grid.
		
		//find out what height the header is...
		var headerHeight = dojo.marginBox(this.headerContainer).h;
		
		if (this.accountForPagingWidgetWhenResettingHeight) {
			if (this.pagingWidget) {
				headerHeight += dojo.marginBox(this.pagingWidget.domNode).h;
			}
		}
		
		dojo.style(this.resizeIndicator, "top", this.pagingWidget.domNode.offsetHeight+"px");

		var moduleMargins = 20;
		// Note by SPC: Unfortunately Task 26230 introduced that 'moduleMargins' setting
		// which results setting the height to 20 less than what was asked for for most
		// uses of the grid ... so for those uses that DON'T have a footer, we make use
		// of the accountForFooterWhenResettingHeight attribute to make sure that the
		// height setting is honored.
		if (!this.accountForFooterWhenResettingHeight) {
			moduleMargins = 0;
		}
		
		//and resize the module to fit
		dojo.style(this.moduleContentPane, "height", Math.max(height-headerHeight-moduleMargins, 50)+"px");
	},

	getHeightNeededToHaveAllRowsShowWithoutScrolling: function() {
		// summary:
		//		Return an integer, representing the appropriate height to use in
		//		a call to setHeight in order to have this grid appear with all
		//		the rows completely visible (i.e. without any scolling for the
		//		rows that are currently displayed).

		var containerNodeHeight = dojo.position(this.module.containerNode).h,
			headerHeight = dojo.position(this.headerContainer).h;

		return headerHeight + containerNodeHeight;
	},
	
	uninitialize: function() {
		if (this._connections) {
			for (var i = 0; i< this._connections.length; i++) {
				dojo.disconnect(this._connections[i]);
			}
			this._connections = null;
		}
		
		this.module.destroyRecursive();
		this.header.destroyRecursive();
		this.pagingWidget.destroyRecursive();
		this.gridmodel.destroyRecursive();
		this.inherited(arguments);
	},

	isPagingWidgetShown: function() {
		return (this.pagingWidget && this.pagingWidget.isShown());
	},
	
/////////////////////////////////////////////////////////////////////////////////
//// Public API

	configureGrid: function(settings) {
		this.gridmodel.configureGrid(settings);
	},
	
	setReadOnly: function (isReadOnly) {
		// summary:
		//		Convenience function for making all cells in the grid
		//		non editable, or editable (depending on the normal grid 
		//		rules)
		this.gridmodel.setAllCellsReadOnly(isReadOnly);
	},
	
	hasRowType: function(rowType) {
		return this.gridmodel.hasRowType(rowType);
	},

	addRowType: function(rowType, settings, parentRowType) {
		//Ensure others can't add internal rowTypes
		if (rowType.indexOf('_') === 0) {
			ErrorHandler.warn("Cannot Add a RowType starting with '_'");
		} else {
			this.gridmodel.addRowType(rowType, settings, parentRowType);
		}
	},
	
	amendRowType: function(rowType, settings) {
		//Ensure others can't amend internal rowTypes
		if (rowType.indexOf('_') === 0) {
			ErrorHandler.warn("Cannot Amend a RowType starting with '_'");
		} else {
			this.gridmodel.amendRowType(rowType, settings);
		}
	},
	
	addMenuItem: function(rowTypeId, groupName, afterInGroup, operation, layout) {
		// summary:
		//		Adds the specified operation for the row id, and position it after
		//		the 'groupName/afterInGroup' item in the layout.
		
		this.gridmodel.addMenuItem(rowTypeId, groupName, afterInGroup, operation, layout);
	},
	
	hasColumn: function(rowTypeId, columnId) {
		return this.gridmodel.hasColumn(rowTypeId, columnId);
	},

	addColumn: function(rowTypeId, columnId, settings) {
		//Ensure others can't add internal columns
		if (columnId.indexOf('_') === 0) {
			ErrorHandler.warn("Cannot Add a Column starting with '_'");
		} else {
			this.gridmodel.addColumn(rowTypeId, columnId , settings);
		}
	},
	
	removeColumn: function(rowTypeId, columnId) {
		//Ensure others can't remove internal columns
		if (columnId.indexOf('_') === 0) {
			ErrorHandler.warn("Cannot Remove a Column starting with '_'");
		} else {
			this.removeCol(columnId);
			this.gridmodel.removeColumn(rowTypeId, columnId);
		}
	},	
	
	hasGrouping: function(groupingId) {
		if (this._groupManager) {
			return this._groupManager.hasGrouping(groupingId);
		}
		//No Group Manager, no grouping.
		return false;
	},
	
	addGrouping: function(settings) {
		
		if (!this._groupManager) {
			this._groupManager = new com.ibm.rdm.web.grid.GroupManager();
			this._groupManager.setGrid(this);
		}
		
		this._groupManager.addGrouping(settings);
	},
	
	addPaging: function(params) {
		// summary:
		//		Shows the paging widget.
		// params:
		//		getTotalPagesFunction:  a function that will return the total number of pages to be displayed. If a total is not available, returning -1 will 
		//		                    give an unterminated list.
		//		getCurrentPageFunction: a function that will return the current page being shown.
		//		gotoPageFunction:       a function that when called will arrange for the display of the given page.
					
		this.pagingWidget.addPaging(params);
		if (this.virtualScrolling) {
			this.pagingWidget.hide(true);
		}
		
	},

	hidePagingWidget: function () {
		dojo.style(this.pagingWidget.domNode, "visibility", "hidden");
	},
	
	showPagingWidget: function () {
		dojo.style(this.pagingWidget.domNode, "visibility", "visible");
	},
	
	setExpandoVisibility: function(rowType, visible) {
		this.gridmodel.setExpandoVisibility(rowType, visible);
		//Change the viewHeader and module.
		this.header.reapplyCols();
		this.module.reapplyCols(rowType);

		if (this.currentArgs) {
			this.resize(this.currentArgs);
		}
	},
	
	setExpandoWidth: function(width) {
		var primary = this.gridmodel.getPrimaryRowTypeId();
		this.gridmodel.setExpandoWidth(width);
		//Change the viewHeader and module.
		this.header.reapplyCols();
		this.module.reapplyCols(primary);

		if (this.currentArgs) {
			this.resize(this.currentArgs);
		}
	},
		
	addToSelection: function(keys, /*optional*/ disableOnChange) {
		// summary:
		//		Adds the rows for the given array of keys to the selection.
		// description:
		//		If a row exists and is not in the selection, it is added.
		//		If it doesn't exist or is already part of the selection, nothing happens.
		//		If the selection is changed, a selection changed is fired.
		//		Note that if the grid does not allow multiselection, only one of the rows given will be selected at the end.
		// keys:
		//		An array of row keys to add to the selection.
		this.module.addToSelection(keys, disableOnChange);		
	},
	
	removeFromSelection: function(keys) {
		// summary:
		//		Removes the rows for the given array of keys from the selection.
		// description:
		//		If a row exists and is in the selection, it is removed.
		//		If it doesn't exist or is already not part of the selection, nothing happens.
		//		If the selection is changed, a selection changed is fired.
		// keys:
		//		An array of row keys to remove from to the selection.
		this.module.removeFromSelection(keys);
	},
	
	clearSelection: function() {
		// summary:
		//		This function clears the current selection in the grid.
		this.module.selectAll(false);
	},
	
	notifyEditStartedOnCell: function(rowKey, columnId) {
		// summary:
		//		Tells the grid that a cell within it is going into
		//		edit mode.  Moves focus to that cell and puts the
		//		row into actionable mode.
		//		Does not, in itself, start any editors on the
		//		row.  May be used by a widget contained in the row
		//		to indicate that it has internally switched to
		//		edit mode.
		// rowKey:
		//		Identifier for the row that is being edited.
		// columnId:
		//		Identifies the cell within the row.
		this.module.notifyEditStartedOnCell(rowKey, columnId);
	},
	
	isInEditMode: function() {
		// returns:
		//		true if the grid is currently in edit mode (Actionable Mode) on a row.
		return this.module.isInEditMode();
	},
	
	onEditorSaveKey: function() {
		// summary:
		//		Called when a save operation is invoked (typically by a hotkey) from a contained editor, or a widget that
		//		can contain an editor.  Notifies the grid that outstanding grid edits should be saved.
		this.module.onEditorSaveKey();
	},
	
	onEditorCancelKey: function() {
		// summary:
		//		Called when a edit cancel operation is invoked  (typically by a hotkey) from a contained editor, or a widget that
		//		can contain an editor.  Notifies the grid that outstanding grid edits should be saved.
		this.module.onEditorCancelKey();
	},

	deleteRow: function(resourceKeydeleted, doNotAnimate) {
		//TODO Make doNotAnimate a gridmodel element.
		this.module.deleteRow(resourceKeydeleted, doNotAnimate);
		this.gridmodel._notifyRowDeletes([resourceKeydeleted]);
	},
	
	deleteRows: function (rows) {
		// summary:
		//		Deletes the rows mentioned.
		// rows:
		//		An array of row keys.
		this.module.deleteRows(rows);
		this.gridmodel._notifyRowDeletes(rows);
	},
	
	isLoaded: function (key) {
		// summary:
		//		Returns whether or not the given key is loaded in the grid.  The key
		//		is loaded if it is currently displayed.	
		return this.module.isLoaded(key);
	},

	scrollIntoView: function (key, /* optional */ visibleCallback) {
		// summary:
		//		Scrolls the given row into view
		// key:
		//		the row key for the desired row
		// visibleCallback:
		//		called when row is visible
		this.module.scrollIntoView(key, visibleCallback);
	},
	
	notifyRowDeletes: function(rowKeys) {},
	
	insertRow: function(insertAfterHereKey, newKey, rowTypeId) {
		// summary:
		//		Allows a row to be inserted.
		// insertAfterHereKey:
		//		The key for the row after which the row will be inserted.
		//		If it is null, the insert happens as the first row.
		// newKey:
		//		The key of the row to insert.
		// rowTypeId:
		//		The rowType of the row to insert.
		// returns:
		//		A deferred that resolves when the row has been inserted and fully displayed.

		//Need to potentially wait for the preDisplayCallback to execute.		
		var deferred = this.preDisplayCallbackFunction && this.preDisplayCallbackFunction();
		if (deferred) {
			var newDeferred = new dojo.Deferred();
			deferred.addBoth(this, function() {
				this.module.insertRow(insertAfterHereKey, newKey, rowTypeId).addBoth(function() {
					newDeferred.callback();
				});
			});
			return newDeferred;
		} else {
			return this.module.insertRow(insertAfterHereKey, newKey, rowTypeId);
		}
	},
	
	insertRows: function(insertAfterHereKey, rows) {
		// summary:
		//		Allows a row to be inserted.
		// insertAfterHereKey:
		//		The key for the row after which the row will be inserted.
		//		If this is null, the row is inserted at the very beginning of the document.
		// rows:
		//		Array of {key: ..., rowtype: ...} entries.

		//This should really return a deferred like the insertRow case
		//Need to potentially wait for the preDisplayCallback to execute.
		var deferred = this.preDisplayCallbackFunction && this.preDisplayCallbackFunction();
		if (deferred) {
			deferred.addBoth(this, function() {
				this.module.insertRows(insertAfterHereKey, rows);
			});
		} else {
			this.module.insertRows(insertAfterHereKey, rows);
		}		
	},
	
	selectGroup: function(groupingId, sortFunction, callbackWithSorted) {
		this.header.set("groupingId", groupingId);
		this.header.render();
		if (!this._groupManager) {
			ErrorHandler.error("Attempt to select a Group without adding it first", groupingId);
		} else {
			this._groupManager.selectGroup(groupingId, sortFunction, callbackWithSorted);
		}
	},

	selectNoGroup: function() {
		this.header.set("groupingId", null);
		this.header.render();
	},
	
	showColumnsDialog: function() {
		this.header.showColumnsDialog();
	},

	//showRelationsDialog: function() {
	//	this.header.showRelationsDialog();
	//},
	
	getRowsOrder: function() {
		return this.module.getRowsOrder();
	},
	
	getRowCount: function() {
		return this.module.getRowCount();
	},
	
	hardClear: function () {
		// summary:
		//		This method is used to differntiate between
		//		a clear of a page and a refresh of the whole
		//		grid, i.e. column headers etc.  It's here
		//		so that if we are supporting virtual scrolling
		//		the virtual grid knows to actually clean itself
		//		up and prepare for new content.
		this._removeAllChangeNotifierListeners();
		this.module.hardClear();
	},
	
	clear: function() {
		// summary:
		//		Clear the contents of the grid
		this.module.clear();
	},
	
	showArtifacts: function(artifacts) {
		//Need to potentially wait for the preDisplayCallback to execute.
		var deferred = this.preDisplayCallbackFunction && this.preDisplayCallbackFunction();
		if (deferred) {
			deferred.addBoth(this, function() {
				this._showArtifacts(artifacts);
			});
		} else {
			this._showArtifacts(artifacts);
		}			
	},
	
	getLastTopVisibleItem: function() {
		// summary:
		//		Get the last visible top row item from the viewport.
		this.lastTopVisibleItem = this.module.getTopVisibleItemFromViewport();
	},

	getLastFocusRowInfo: function() {
		// summary:
		//		Returns a property list detailing information about the current focus
		//		row (if a row currently has focus).

		return (this.lastFocusRowInfo = this.module.getFocusRowInfoFromViewport());
	},
	
	getLastBottomVisibleItem: function() {
		// summary:
		//		Get the last visible bottom row item from the viewport.
		this.lastBottomVisibleItem = this.module.getBottomVisibleItemFromViewport();
	},
	
	_applySelectionAndScrollPosition: function() {
		// summary:
		//		Apply last scroll position and the selected artifacts
		if (this.currentArgs && !this.virtualScrolling) {
			this.resize(this.currentArgs);
		}
		
		// Re-apply the scroll position using the last visible top row.
		// If top row is not available, use the last visible bottom row.
		// If bottom row is not available, scroll to the top of the module.
		if (this.virtualScrolling) {
			// reset flag used to indicate whether we are done scrolling to the position.
			// If done, 'reset' will set to true and last top and bottom visible items will be reset.
			var reset = false,
				rowViewportTopOffset = this.lastFocusRowInfo ? this.lastFocusRowInfo.rowViewportTopOffset : undefined,
				lastFocusRowItem = this.lastFocusRowInfo ? this.lastFocusRowInfo.key : null,
				itemToScrollIntoView = lastFocusRowItem || this.lastTopVisibleItem;
			if (itemToScrollIntoView) {
				this.scrollIntoView(itemToScrollIntoView);
				var row = this.getRowByKey(itemToScrollIntoView);
				if (row && row.node) {
					this.module.scrollNodePosition(
						row.node, lastFocusRowItem ? false : true,
						false, rowViewportTopOffset
					);
					reset = true;
				}
				else if (this.lastBottomVisibleItem) {
					this.scrollIntoView(this.lastBottomVisibleItem);
					row = this.getRowByKey(this.lastBottomVisibleItem);
					if (row && row.node) {
						this.module.scrollNodePosition(row.node, false, false, rowViewportTopOffset);
						reset = true;
					}
				}
			}
			else if (this.lastBottomVisibleItem) {
				this.scrollIntoView(this.lastBottomVisibleItem);
				row = this.getRowByKey(this.lastBottomVisibleItem);
				if (row && row.node) {
					this.module.scrollNodePosition(row.node, false, false, rowViewportTopOffset);
				}
				reset = true;
			}
			if (reset) {
				this.lastTopVisibleItem = null;
				this.lastBottomVisibleItem = null;
			}
			this.lastFocusRowInfo = null;
		}
		
		// Re-select the selected artifacts if any
		if (this.module.selectedArtifactsUri && this.module.selectedArtifactsUri.length > 0) {
			while (this.module.selectedArtifactsUri.length > 0) {
				var uri = this.module.selectedArtifactsUri.pop();
				this.addToSelection([uri]);
			}
		}
	},
	
	_getCollapsedExpandedList: function(artifactState) {
		// summary:
		//		Get the expanded and collapsed sections for the module.
		var uris = [];
		if (this.module.moduleStructure) {
			if (artifactState == ArtifactState.COLLAPSED) {
				// Get the collapsed section
				if (this.module.collapsedArtifactUris && this.module.collapsedArtifactUris.length > 0) {
					while (this.module.collapsedArtifactUris.length > 0) {
						var uri = this.module.collapsedArtifactUris.pop();
						var isCollapsed = this.module.moduleStructure.isCollapsedSection(uri);
						// Skip if the artifact is already collapsed.
						if (!isCollapsed) {
							uris.push(uri);
						}
					}
				}
			}
			else if (artifactState == ArtifactState.EXPANDED) {
				// Get the expanded section
				if (this.module.expandedArtifactUris && this.module.expandedArtifactUris.length > 0) {
					while (this.module.expandedArtifactUris.length > 0) {
						var uri = this.module.expandedArtifactUris.pop();
						var isCollapsed = this.module.moduleStructure.isCollapsedSection(uri);
						// Skip if the artifact is already expanded.
						if (isCollapsed) {
							uris.push(uri);
						}
					}
				}
			}
			else if (artifactState == ArtifactState.BOTH) {
				// Get both expanded and collapsed sections
				if (this.module.expandedArtifactUris && this.module.expandedArtifactUris.length > 0) {
					while (this.module.expandedArtifactUris.length > 0) {
						var uri = this.module.expandedArtifactUris.pop();
						var isCollapsed = this.module.moduleStructure.isCollapsedSection(uri);
						// Skip if the artifact is already expanded.
						if (isCollapsed) {
							uris.push(uri);
						}
					}
				}
				if (this.module.collapsedArtifactUris && this.module.collapsedArtifactUris.length > 0) {
					while (this.module.collapsedArtifactUris.length > 0) {
						var uri = this.module.collapsedArtifactUris.pop();
						var isCollapsed = this.module.moduleStructure.isCollapsedSection(uri);
						// Skip if the artifact is already collapsed.
						if (!isCollapsed) {
							uris.push(uri);
						}
					}
				}
			}
			uris.reverse();
		}
		return uris;
	},
	
	_addChangeNotifierListener: function(listenerObject) {
		ChangeNotifier.addListener(listenerObject);
		this._changeNotifierListeners.push(listenerObject);
		return listenerObject;
	},

	_removeChangeNotifierListener: function(listenerObject) {
		ChangeNotifier.removeListener(listenerObject);
		var index = this._changeNotifierListeners.indexOf(listenerObject);
		if (index >= 0) {
			this._changeNotifierListeners.splice(index, 1);
		}
	},

	_removeAllChangeNotifierListeners: function() {
		var length = this._changeNotifierListeners.length,
			i;
		for (i = 0; i < length; i++) {
			ChangeNotifier.removeListener(this._changeNotifierListeners[i]);
		}
		this._changeNotifierListeners = [];
	},
	
	_applyCollapsedExpandedSection: function(artifactState) {
		// summary:
		//		Apply the expanded and collapsed section for the module.
		var uris = this._getCollapsedExpandedList(artifactState);
		var doApply = false;
		if (uris.length > 0) {
			doApply = true;
			// Re-apply collapsed and/or expanded state of artifacts.
			var reconciledListener = this._addChangeNotifierListener({
				eventTopic: com.ibm.rdm.web.notifications.EventTopics.MODULE_GRID_RECONCILED,
				callbackFunction: function(){
					if(this.restoreAfterRefresh) {
						this._removeChangeNotifierListener(reconciledListener);
						// Re-select the selected artifacts if any and the last scroll position
						this._applySelectionAndScrollPosition();
						this.restoreAfterRefresh = false;
					}
				},
				callbackScope: this
			});
			this.module.moduleStructure.collapseOrExpandSectionList(uris);
		}
		if (!doApply) {
			// Re-select the selected artifacts if any and the last scroll position
			this._applySelectionAndScrollPosition();
		}
	},
	
	_showArtifacts: function(artifacts) {

		var showArtifactsImpl = dojo.hitch(this, function () {
			this.module.clear();
			
			this.pagingWidget.refresh();
	
			this.header.render();
			this.module.applyOrder(artifacts);
			
			if (this.module.moduleStructure) {
				this.restoreAfterRefresh = false;
				var currentModuleTreeLevel = this.module.moduleStructure.getCurrentModuleTreeLevel();
				if (currentModuleTreeLevel == -1) {
					currentModuleTreeLevel = 999;
				}
				if (this.isRefresh) {
					// Re-apply the module levels to display
					this.module.moduleStructure.collapseAllGreaterThan(currentModuleTreeLevel);
					var reconciledListener = this._addChangeNotifierListener({
						eventTopic: com.ibm.rdm.web.notifications.EventTopics.MODULE_GRID_RECONCILED,
						callbackFunction: function(){
							this._removeChangeNotifierListener(reconciledListener);
							var maxDepth = this.module.moduleStructure.getDeepestLevel();
							this.restoreAfterRefresh = true;
							if(currentModuleTreeLevel == 999 || currentModuleTreeLevel == maxDepth) {
								// Re-apply collapsed artifacts if any
								this._applyCollapsedExpandedSection(ArtifactState.COLLAPSED);
							}
							else if (currentModuleTreeLevel == 1) {
								// Re-apply expanded artifacts if any
								this._applyCollapsedExpandedSection(ArtifactState.EXPANDED);
							}
							else {
								// Re-apply both expanded and collapsed artifacts if any
								this._applyCollapsedExpandedSection(ArtifactState.BOTH);
							}
						},
						callbackScope: this
					});
					this.isRefresh = false;
				}
				else {
					if (this.currentArgs && !this.virtualScrolling) {
						this.resize(this.currentArgs);
					}
				}
			}
			else {
				if (this.currentArgs && !this.virtualScrolling) {
					this.resize(this.currentArgs);
				}
			}
		});
		
		if (this._started) {
			showArtifactsImpl();
		} else {
			var con = dojo.connect(this, "startup", this, function () {
				dojo.disconnect(con);
				// make sure references to projectOperations is set by RSGV before
				// building the grid DOM, which will ask it whether things are
				// editable.
				showArtifactsImpl();
			});
		}
	},

	setHighlightFunction: function(functionName, func) {
		this.module.setHighlightFunction(functionName, func);
	},
	
	setAddToNewFunction: function(addToNewFunction) {
		this._addToNewArtifacts = addToNewFunction; 
	},
	
	scrollHeaderToMatchContent : function () {
		// summary :
		// programatically scroll the header the same amount as
		// the content.
		var amnt = this.moduleContentPane.scrollLeft;
		this.headerContainer.scrollLeft = amnt;
	},

	refreshRows: function(rowDetails, /*Boolean?*/ noFocus, /*Boolean?*/deferDestroy) {
		// summary:
		//		Allows the refreshing of rows.
		// rowDetails:
		//		Array of {key: rowkey, rowtype: rowtype} objects describing each row to refresh.
		// noFocus:
		//		If set and true, no attempt is made to restore focus to the grid
		//		following the refresh.
		// deferDestroy:
		//		If set and true, old rows are not destroyed and are put into a bucket.
		//		Solves an issue that can occur in IE when editing multiple attributes on the same row.
		return this.module.refreshRows(rowDetails, noFocus, deferDestroy);
	},
	
	runRowClassFunctions: function(rowDetails) {
		// summary:
		//		Runs the registered row class functions for the given rows.
		// rowDetails:
		//		An array of row identifiers.
		this.module.runRowClassFunctions(rowDetails);
	},
	
	refreshExpandos: function(rowDetails) {
		// summary:
		//		Allows the refreshing of rows.
		// rowDetails:
		//		Array of {key: rowkey, rowtype: rowtype} objects describing each row to refresh.
		this.module.refreshExpandos(rowDetails);
	},
	
	removeCol: function(columnId) {
		var primaryRowTypeId = this.gridmodel.getPrimaryRowTypeId();
		
		this.gridmodel.setVisible(primaryRowTypeId, columnId, false);
		
		this.header.removeCol(columnId);
		this.module.removeCol(columnId);

		ChangeNotifier.notifyListeners({
			eventTopic: com.ibm.rdm.web.notifications.EventTopics.GRID_COLUMNS_MODIFIED
		});
		
		if (this.currentArgs) {
			this.resize(this.currentArgs);
		}
	},
	
	hasCol: function(rowId, columnId) {
		
		return this.gridmodel.columnExists(rowId, columnId);
		
	},


	addCol: function(columnId) {
		//Need to potentially wait for the preDisplayCallback to execute.
		//Need to potentially wait for the preDisplayCallback to execute.
		if (this.module.moduleStructure) {
			this.module.moduleStructure = null;
		}
		var deferred = this.preDisplayCallbackFunction && this.preDisplayCallbackFunction();
		if (deferred) {
			deferred.addBoth(this, function() {
				this._addCol(columnId);
			});
		} else {
			this._addCol(columnId);
		}	
	},
	
	_addCol: function(columnId) {
		var primaryRowTypeId = this.gridmodel.getPrimaryRowTypeId();
		
		this.gridmodel.setVisible(primaryRowTypeId, columnId, true);

		var cols = this.gridmodel.getColumnIds(primaryRowTypeId, columnId);
		
		//Get the the position of the col before from the visible cols
		var position = 0;
		var finalPosition;
		var self = this;
		dojo.forEach(cols, function(item) {
			if (item === columnId) {
				finalPosition = position;
			}
			if (self.gridmodel.isVisible(primaryRowTypeId, item)) {
				position++;
			}
		});
		
		this.header.addCol(primaryRowTypeId, columnId, finalPosition);
		this.module.addCol(primaryRowTypeId, columnId, finalPosition);

		ChangeNotifier.notifyListeners({
			eventTopic: com.ibm.rdm.web.notifications.EventTopics.GRID_COLUMNS_MODIFIED
		});

		if (this.currentArgs) {
			this.resize(this.currentArgs);
		}
	},
	
	getUserCols: function(rowType) {
		// summary:
		//		Gets all the column info for the given rowType.
		// returns:
		//		Returns an array of {columnId: id, visible: true/false, width: newWidth}.
		
		return this.gridmodel.getUserCols(rowType);
	},
	
	setUserCols: function(rowType, columnDetails) {
		// summary:
		//		Sets the visible cols that the user is in control of, as an array of {colId: ... , width: ...}.
		//		These are reordered and moved to the front of the collection, and made visible.
		//		All the rest are made invisible.
		//		Ignores ones it cannot find.
		
		this.gridmodel.setUserCols(rowType, columnDetails);

		//Change the viewHeader and module.
		this.header.reapplyCols();
		this.module.reapplyCols(rowType);

		if (this.currentArgs) {
			this.resize(this.currentArgs);
		}
	},
	
	reorderCols: function(rowType, newCols) {
		//Allows the re-odering, resizing and changing visibility of cols.
		// newCols: an array of {columnId: id, visible: true/false, width: newWidth}.
		// It will not allow alwaysVisible columns to become visible, or fixedWidth columns to change width.
		// It will ignore columnIds it knows nothing about.

		//Update the gridmodel
		var gm = this.gridmodel;
		
		gm.reorderColumns(rowType, newCols);
		var self = this;
		dojo.forEach(newCols, function(item) {
			gm.setWidth(rowType, item.columnId, item.width.value, item.width.set);
			gm.setVisible(rowType, item.columnId, item.visible.checked);
		});
		
		
		if(this._reorderNotification) {
			this._reorderNotification(dojo.hitch(this, this._postReorderCols, rowType), newCols);
		} else {
			this._postReorderCols(rowType);
		}
	},

	_postReorderCols: function(rowType) {
		//Change the viewHeader and module.
		this.header.reapplyCols();
		this.module.reapplyCols(rowType);

		if (this.currentArgs) {
	//		this.resize(this.currentArgs);
		}
	},
	
	showMessage: function(messageToDisplay) {
		
		this.module.clear();
	
		this.header.render();
		this.message=messageToDisplay;
		this.pagingWidget.refresh();
		
		this.module.applyOrder([{key: "message"+(message_count++), rowtype: MESSAGE_ROW}]);	
	},
	
	focus: function() {
		// summary:
		//		Take focus.  Focus is moved to the row/cell that was last in
		//		focus, or set for focus.
		this.module.focus();
	},

	retakeFocus: function() {
		// summary:
		//		Take focus.  Focus is moved to the row/cell that was last in
		//		focus, or set for focus.

		this.module.retakeFocus();
	},

	simulateTabIntoGrid: function() {
		// summary:
		//		For details, please see the documentation for a method by this
		//		same name in the Module class.

		this.module.simulateTabIntoGrid();
	},
	
	moveFocusToCellOnRow: function(/*String*/rowKey) {
		// summary:
		//		Moves the focus cell of the grid to a cell in the row indentified by
		//		rowKey.  The column chosen is the column that was last focussed on that
		//		type of row - typically this will be the same column that was previously
		//		in focus on another row.  If a cell cannot be determinied, the whole
		//		row is given focus.
		//		Will take focus if possible, but if the row is not yet displayed, still
		//		sets the grid such that the next time it takes focus, the given cell will
		//		be focused.
		// rowKey:
		//		Identifier for the row on which to focus.
		// returns:
		//		true if focus was taken.
		return this.module.moveFocusToCellOnRow(rowKey);
	},
	
	setFocusRowById: function(id, skipBrowserFocus) {
		// summary:
		//		Set the given row to the current selected row.  If skipFocus is true,
		//		browser focus will not be set to the row.
		return this.module.selectObjectById(id, skipBrowserFocus);
	},

	getFocusRowKey: function() {
		var key = null;
		if (this.module.getFocusRow) {
			var focusRow = this.module.getFocusRow();
			if (focusRow) {
				key = focusRow.key;
			}
		}
		return key;
	},
	
	getSelectedObjectId: function() {
		return this.module.getSelectedObjectId();
	},
	
	getSelectedRows: function(){
		// summary:
		//		Returns an array of selected objects of the form 
		//		{rowTypeId: ..., uri: ... }.
	
		return this.module.getSelectedRows();
	},
	
	containsRow: function (key) {
		// summary:
		//		Is the row identified by key in this grid ?
		return this.module.allRows[key];
	},
	
	viewHasBeenRestored: function(){
		// summary:
		//		Notification that we are in a view that
		//		has been restored from and off-screen cache.
		if (this.module.viewHasBeenRestored) {
			this.module.viewHasBeenRestored();
		}
	},
	
	isColumnShowing: function(colId) {
		var found = false;
		var gm = this.gridmodel;
		var primaryRowType = gm.getPrimaryRowTypeId();
		dojo.forEach(this.getUserCols(primaryRowType), function(col) {
			if (col.colId === colId) {
				found = true;
			}
		});
		return found;
	},
	
	scrollGoToPosition: function(key, listener) {
		// Scroll the GoTo item to the view.
		if (this.virtualScrolling) {
			this.scrollIntoView(key);
			var row = this.getRowByKey(key);
			if (row && row.node) {
				// Force the GoTo item as the top item in the viewport
				// to ensure the item always in the viewport
				this.module.setRowAsTopItemInViewport(row.node);
				// Scroll the row back to the view 
				this.module.scrollNodePosition(row.node, true, true);
				if (listener) {
					// Remove RESET_GOTO_ITEM event listener when we are able to scroll the row into the view
					ChangeNotifier.removeListener(listener);
				}
			}
			// Set last top visible item as the GoTo item to in sync with the top item in the viewport
			this.lastTopVisibleItem = key;
		}
	},
	
	setGoToItemAsTopItem: function(key) {
		// Force the GoTo item as the top item in the viewport.
		var row = this.getRowByKey(key);
		if (row && row.node) {
			this.module.setRowAsTopItemInViewport(row.node);
		}
	},
	
	//Search API
	
	_getSearchColumns: function (key, searchTerm, selectClassName, markClassName, replaceClassName) {
		//If we have a search in progress, clear it.
		if (this._searchDetails) {
			this.clearFind();
		}
		
		//Remember search detail
		this._searchDetails = {key: key, searchTerm: searchTerm, selectClassName: selectClassName, markClassName: markClassName, replaceClassName: replaceClassName};

		//Find all columns supporting find, in the right order.  We only search on visible columns.
		var searchCols = [];
		var gm = this.gridmodel;
		var primaryRowType = gm.getPrimaryRowTypeId();
		dojo.forEach(this.getUserCols(primaryRowType), function(col) {
			if (gm.supportsFind(primaryRowType, col.colId)) {
				searchCols.push(col.colId);
			}
		});
//		ErrorHandler.debug("Searching " + searchCols.length + " cols ");
		return searchCols;
	},
	
	findNextTerm: function (key, searchTerm, selectClassName, markClassName, replaceClassName, done, noMore) {
		// summary:
		//		Identifies the collection, marks all with the mark class name, and the first with the select class name.
		//		Only one search can be ongoing at a time (per grid), so calling this will clear all previous results (if not already cleared).
		// key:
		//		The key for the row you want to search.
		// searchTerm:
		//		The term being searched for. (At the moment, a string. May become a more complex type).
		// selectClassName:
		//		The (string) class used to mark-up the one 'selected' item.
		// markClassName:
		//		The (string) class used to mark-up all found items in the row.
		// replaceClassName:
		//		The (string) class used to mark-up a replaced item in the row.
		// done:
		//		Function to call back when the initial mark-up is complete, and the first is selected.
		// noMore:
		//		Function to call back if there are no mark-ups found.

		var searchCols = this._getSearchColumns(key, searchTerm, selectClassName, markClassName, replaceClassName);
		//Tell them all to markup,
		var dl = [];
		var result;
		var gm = this.gridmodel;
		var primaryRowType = gm.getPrimaryRowTypeId();
		dojo.forEach(searchCols, function(colId) {
			result = gm.findMarkAllNext(primaryRowType, colId, key, searchTerm, markClassName);
			if (result) {dl.push(result);}
		});
		
		if (searchCols.length === 0) {
			noMore();
		} else {
			this._searchDetails.cols = searchCols;
			this._searchDetails.primaryRowType = primaryRowType;
			this._searchDetails.currCol = 0;
			
			//Select next on the first one - but we have to wait for deferred
			if (dl.length > 0) {
				var deflist = new dojo.DeferredList(dl);
				deflist.onCallback(function() {
					this.nextFind(done, noMore);
				});
			} else {
				this.nextFind(done, noMore);
			}
		}
	},
	
	findPrevTerm: function (key, searchTerm, selectClassName, markClassName, replaceClassName, done, noMore) {
		// summary:
		//		Identifies the collection, marks all with the mark class name, and the last with the select class name.
		//		Only one search can be ongoing at a time (per grid), so calling this will clear all previous results (if not already cleared).
		// key:
		//		The key for the row you want to search.
		// searchTerm:
		//		The term being searched for. (At the moment, a string. May become a more complex type).
		// selectClassName:
		//		The (string) class used to mark-up the one 'selected' item.
		// markClassName:
		//		The (string) class used to mark-up all found items in the row.
		// replaceClassName:
		//		The (string) class used to mark-up a replaced item in the row.
		// done:
		//		Function to call back when the initial mark-up is complete, and the first is selected.
		// noMore:
		//		Function to call back if there are no mark-ups found.

		var searchCols = this._getSearchColumns(key, searchTerm, selectClassName, markClassName, replaceClassName);
		//Tell them all to markup,
		var dl = [];
		var result;
		var gm = this.gridmodel;
		var primaryRowType = gm.getPrimaryRowTypeId();
		dojo.forEach(searchCols, function(colId) {
			result = gm.findMarkAllPrev(primaryRowType, colId, key, searchTerm, markClassName);
			if (result) {dl.push(result);}
		});
		
		if (searchCols.length === 0) {
			noMore();
		} else {
			this._searchDetails.cols = searchCols;
			this._searchDetails.primaryRowType = primaryRowType;
			this._searchDetails.currCol = searchCols.length - 1;
			
			//Select prev on the last one - but we have to wait for deferred
			if (dl.length > 0) {
				var deflist = new dojo.DeferredList(dl);
				deflist.onCallback(function() {
					this.prevFind(done, noMore);
				});
			} else {
				this.prevFind(done, noMore);
			}
		}
	},

	nextFind: function (done, noMore) {
		// summary:
		//		Selects the next item. (If no 'findTerm' has been called, just writes an error log).
		// done:
		//		Function to call back when next item is highlighted.
		// noMore:
		//		Function to call back if there are no more items to be highlighted.
		
		if (!this._searchDetails) {
//			ErrorHandler.debug("Error: nextFind called with no search established");
		} else {
			var sd = this._searchDetails;
			var self = this;
			
			this.gridmodel.nextFind(sd.primaryRowType, sd.cols[sd.currCol], sd.key, sd.selectClassName, sd.replaceClassName,
				//done function - this is done. Just pass the call on
				done, 
				//noMore function - must search on further cols in the row if this col is finished
				function() {
					if (++sd.currCol >= sd.cols.length) {
						//No more cols - callback nomore
						//adjust the number back
						sd.currCol--;
						noMore();
					} else {
						//Search the next col  (incrementing the currCol has moved to next)
						self.nextFind(done, noMore);
					}
				}
			);
		}
	},
	
	prevFind: function (done, noMore) {
		// summary:
		//		Selects the previous item.  (If no 'findTerm' has been called, just writes an error log).
		// done:
		//		Function to call back when previous item is highlighted.
		// noMore:
		//		Function to call back if there are no previous items to be highlighted.

		if (!this._searchDetails) {
//			ErrorHandler.debug("Error: prevFind called with no search established");
		} else {
			var sd = this._searchDetails;
			var self = this;
			
			this.gridmodel.prevFind(sd.primaryRowType, sd.cols[sd.currCol], sd.key, sd.selectClassName, sd.replaceClassName,
				//done function - this is done. Pass the callback on
				done, 
				//noMore function - must search on further cols in the row if this col is finished
				function() {
					if (--sd.currCol < 0) {
						//No more cols - callback nomore
						sd.currCol++;
						noMore();
					} else {
						//Search the previous col (decrementing the currCol has moved to prev)
						self.prevFind(done, noMore);
					}
				}
			);
		}
	},
	
	replaceFind: function (replaceTerm, onDone) {
		// summary:
		//		Replaces the current find term with the given string.
		// replaceTerm:
		//		replacement string
		// onDone:
		//		Function to call back when replacement done.

		if (!this._searchDetails) {
//			ErrorHandler.debug("Error: replaceFind called with no search established");
		} else {
			var sd = this._searchDetails;
			this.gridmodel.replaceFind(sd.primaryRowType, sd.cols[sd.currCol], sd.key, replaceTerm, sd.replaceClassName, [sd.selectClassName, sd.markClassName], onDone); 
		}
	},
	
	clearFind: function () {
		// summary:
		//		Removes all markings.  (If no 'findTerm' has been called (therefore no markings in place), does nothing).
		if (!this._searchDetails) {
			//ErrorHandler.debug("Warning: clearFind called with no search established");
		} else {
			var gm = this.gridmodel;
			var sd = this._searchDetails;
			dojo.forEach(sd.cols, function(colId) {
				gm.clearFind(sd.primaryRowType, colId, sd.key);
			});
		}
	},
	
	getCurrentFindNode: function() {
		// summary:
		//		Returns the node representing the current find term.
		
		var sd = this._searchDetails;
		var colId = sd.cols[sd.currCol];
		return this.gridmodel.getCurrentFindNode(sd.primaryRowType, colId, sd.key);
	},
	
	getCellHtml: function(key, colId) {
		return this.module.getCellHtml(key, colId);
	},
	
	getRowByKey: function(key){
		return this.module.getRowByKey(key);
	},
	
	getKeyPressHandlers: function(){
		return this.gridmodel.getKeyPressHandlers();
	},

	getKeyDownHandlers: function(){
		return this.gridmodel.getKeyDownHandlers();
	},
	
	addToNewArtifacts: function(artifactUri){
		if (dojo.isFunction(this._addToNewArtifacts)){
			this._addToNewArtifacts(artifactUri);
		}
	},
	
	getSelectedArtifactsUri: function() {
		// summary:
		//		Get the current selected artifacts.
		//		so that we can restore the selected artifacts after refresh or view changed.
		this.module.selectedArtifactsUri = [];
		var selectedRows = this.getSelectedRows();
		for (var i=0; i < selectedRows.length; i++) {
			var row = selectedRows[i];
			this.module.selectedArtifactsUri.push(row.uri);
		}
	},
	
	prepareRestoreValues: function(isUpdateNeeded, /*Boolean?*/avoidCapturingExpansionStates) {
		// summary:
		//		Prepare data for restoring after module refresh or view changed.
		//		Data such as top and bottom visible items, collapsed 
		//		and expanded sections and the current selected items.
		// isUpdateNeeded:
		//		Boolean flag to indicate whether the restore data need to be updated.
		// avoidCapturingExpansionStates:
		//		By default, row expansion states will be captured.  Pass as true to avoid
		//		capturing these states (for example, you should pass as true when
		//		resurrecting a saved filter that contains a 'hierarchyLevel' to display
		//		[which is used, for example, in our "Configure Page Settings" grid
		//		header drop down menu to display something like "Show 1 Level"]).
		
		// Check if the restore values are already prepared.
		if (!isUpdateNeeded && 
			((this.module.collapsedArtifactUris && this.module.collapsedArtifactUris.length > 0) ||
			(this.module.expandedArtifactUris && this.module.expandedArtifactUris.length > 0) ||
			this.lastTopVisibleItem || this.lastBottomVisibleItem)) {
			return;
		}
			
		if (this.module.moduleStructure){
			this.module.collapsedArtifactUris = [];
			this.module.expandedArtifactUris = [];
			if (!avoidCapturingExpansionStates) {
				this.module.moduleStructure.getCollapsedAndExpandedArtifactUris(
					this.module.collapsedArtifactUris, this.module.expandedArtifactUris);
			}
			this.getLastTopVisibleItem();
			this.getLastBottomVisibleItem();
			this.getLastFocusRowInfo();
			this.getSelectedArtifactsUri();
		}
	},

	
/////////////////////////////////////////////////////////////////////////////////
//// Private Support Methods

	_onSaveKey: function(returnDeferred) {
		if (returnDeferred) {
			return this.gridmodel.onSaveKey();
		}
		this.gridmodel.onSaveKey();
	},
	
	_onEscKey: function() {
		this.gridmodel.onEscKey();
	},
	
	_onCtrlDKey: function(rowTypeId, columnId, objectId, selection) {
		this.gridmodel.onCtrlDKey(rowTypeId, columnId, objectId, selection);
	},
	
	
	
	_keyPressHandler: function(event) {
		// summary:
		//		Generic keypress handler which calls handlers for keys as defined in the
		//		grid model.
		// returns:
		//		true to indicate that the key has *not* been handled and so should be
		//		propagated.

//		console.debug("Grid._keyPressHandler() keyChar: "+ event.keyChar + " keyCode: "+ event.keyCode + " charCode: "+ event.charCode + " keyIdentifier: " + event.keyIdentifier +
//					 " ctrl: " + event.ctrlKey + " shift: " + event.shiftKey + " alt: " + event.altKey);

		return this._keyHandler(event, this.gridmodel.getKeyPressHandlers());
	},
	
	_keyDownHandler: function(event) {
		// summary:
		//		Generic keypress handler which calls handlers for keys as defined in the
		//		grid model.
		// returns:
		//		true to indicate that the key has *not* been handled and so should be
		//		propagated.

//		console.debug("Grid._keyDownHandler() keyChar: "+ event.keyChar + " keyCode: "+ event.keyCode + " charCode: "+ event.charCode + " keyIdentifier: " + event.keyIdentifier +
//					 " ctrl: " + event.ctrlKey + " shift: " + event.shiftKey + " alt: " + event.altKey);

		return this._keyHandler(event, this.gridmodel.getKeyDownHandlers());
	},
	
	_keyHandler: function(event, keyHandlers){
		// summary:
		//		Generic keypress handler that can deal with key events as defined in the
		//		provided keyHandlers.
		// returns:
		//		true to indicate that the key has *not* been handled and so should be
		//		propagated.
	
		var len, handler, eventKeyChar;			
		//console.debug("Grid._keyHandler() keyChar: "+ event.keyChar + " keyCode: "+ event.keyCode + " charCode: "+ event.charCode + " synthetic: " + event.synthetic + " keyIdentifier: " + event.keyIdentifier +
		//			 " ctrl: " + event.ctrlKey + " shift: " + event.shiftKey + " alt: " + event.altKey);
			
		if (!keyHandlers || this._lastHandledEvent === event) {
			return true;
		}
		
		// synthetic events (those we have generated artificially) have a syntheticKeyChar instead of a keyChar.
		eventKeyChar = event.synthetic ? event.syntheticKeyChar : event.keyChar;
		if (dojo.isMac
			&& event.keyCode !== dojo.keys.ENTER
			&& event.keyCode !== dojo.keys.DELETE
			&& event.keyCode !== dojo.keys.BACKSPACE
			&& event.keyCode !== dojo.keys.RIGHT_ARROW
			&& event.keyCode !== dojo.keys.LEFT_ARROW
			) {
			// Unlike other platforms, MacOS doesn't give us the char in upper
			// case, so we take care here to upcase it.
			if (eventKeyChar && event.shiftKey) {
				eventKeyChar = eventKeyChar.toUpperCase();
			}
			// Unlike other platforms, MacOS doesn't provide a keyChar when the
			// Option key (i.e. "Alt" for other browsers) is used, so we take
			// care here to glean the char involved from the keyCode.
			else if (event.keyCode && !event.keyChar) {
				eventKeyChar = String.fromCharCode(event.keyCode).toLowerCase();
			}
		}
		len = keyHandlers.length;
		for (var i = 0 ; i < len ; i++) {
			handler = keyHandlers[i];
			if (
				(
						// Look for a matching keyChar or keyCode.  Only look at keyCode if there is no
						// keyChar in the event because there are keyCodes that can appear both with and
						// without a keyChar.  e.g. "q" and F2 both have keyCode == 113.
						(handler.keyChar && ((dojo.isMac ? handler.keyChar.toLowerCase() : handler.keyChar) === eventKeyChar)) ||
						(!eventKeyChar && handler.keyCode && (handler.keyCode === event.keyCode))
				) &&
				(
						// Look for matching modifier keys.
						// If the handler has copyKey set, instead of looking at event.ctrlKey we compare handler.ctrl with either
						// event.ctrlKey or event.metaKey, depending on the platform (as indicated in COPYKEY_MODIFIER).  This allows
						// mac to use it's meta key rather than ctrl for some operations.
						(handler.copyKey ? (!!handler.ctrl === !!event[COPYKEY_MODIFIER]) : (!!handler.ctrl === !!event.ctrlKey) ) &&
						(!!handler.shift === !!event.shiftKey) &&
						(!!handler.alt === !!event.altKey)
				)
				
			) {
				this._lastHandledEvent = event;
				var focusRow = this.module.getFocusRow();
				if (!focusRow) {
					return true;
				}
				var operations = this.gridmodel.getOperations(focusRow.rowType);
				var operation = operations[handler.operationName];
				if (!operation) {
					// No matching operation - nothing to do
					return true;
				}
				if (!handler.validForEditMode && this.module.isInEditMode()) {
					// Grid is in edit mode, so not valid to use this key. Let it
					// propagate in case it has a function in the editor.
					return true;
				}
				var rowIdsAndTypes = {};
				if (handler.multiRow) {
					// Do the operation on the focus row and all selected rows.
					var selectedRows = this.module.getSelectedRows(true /*includeFocusRow*/),
						count = selectedRows.length;

					// convert selected rows to a map of rowKey->rowType
					for (var j = 0 ; j < count ; j++ ) {
						var row = selectedRows[j];
						rowIdsAndTypes[row.uri] = row.rowTypeId;
					}
					handler.handlerFunction(focusRow.key, rowIdsAndTypes, count, operation);
				} else {
					// Do the operation on the focus row only.
					// 78535: Ensure the sibling has been saved before creating a new artifact from it
					if (focusRow.key.indexOf("STUB_") > -1) {
						var saveDeferred = this.module.onSaveKey(true);
						saveDeferred.then(dojo.hitch(this, function(result) {
							var focusRow = this.module.getFocusRow();
							rowIdsAndTypes[focusRow.key] = focusRow.rowType;
							handler.handlerFunction(focusRow.key, rowIdsAndTypes, 1, operation);
						}));
					} else {
						rowIdsAndTypes[focusRow.key] = focusRow.rowType;
						handler.handlerFunction(focusRow.key, rowIdsAndTypes, 1, operation);
					}
				}
				return;
			}
		}
		return true;
	},
	
	
	_reapplyGridDialogData: function(griddata) {
		var rowType = this.gridmodel.getPrimaryRowTypeId();
		this.reorderCols(rowType, griddata);
	},
	
	measureRows: function () {
		if (this.virtualScrolling) {
			this.module.measureAll();
		}
	},

	_updateCssClassAttributesToShowPaging: function() {
		// summary:
		//		Add appropriate styling to enable consumers of this class to
		//		know when the paging widget is displayed.

		dojo.addClass(this.domNode, "paging-widget-visible");
		dojo.removeClass(this.domNode, "paging-widget-hidden");
	},

	_updateCssClassAttributesToHidePaging: function() {
		// summary:
		//		Add appropriate styling to enable consumers of this class to
		//		know when the paging widget is hidden.

		dojo.removeClass(this.domNode, "paging-widget-visible");
		dojo.addClass(this.domNode, "paging-widget-hidden");
	}
});

com.ibm.rdm.web.grid.Grid.VARIABLE_CHECKBOX = "a";
com.ibm.rdm.web.grid.Grid.VARIABLE_NUMBERBOX = "b";

})();
