/*
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2014, 2015.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 */
(function(){

dojo.provide("com.ibm.rdm.web.mergevvc.VVCArtifactMergeWizardStep");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("jazz.ui.wizard.WizardStep");

dojo.require("com.ibm.rdm.web.common.MessageDisplayWidget");
dojo.require("com.ibm.rdm.web.common.MessageHandler");
dojo.require("com.ibm.rdm.web.compare.CompareConstants");
dojo.require("com.ibm.rdm.web.compare.compareLinksUtils");
dojo.require("com.ibm.rdm.web.compare.compareutils");
dojo.require("com.ibm.rdm.web.comparevvc.CompareArtifactSelectorWidget");
dojo.require("com.ibm.rdm.web.comparevvc.InterimResultClient");
dojo.require("com.ibm.rdm.web.comparevvc.vvccompareutils");
dojo.require("com.ibm.rdm.web.comparevvc.VVCCompareConstants");
dojo.require("com.ibm.rdm.web.merge.ArtifactMergeWidget");
dojo.require("com.ibm.rdm.web.merge.mergeutils");
dojo.require("com.ibm.rdm.web.merge.TagMergeWidget");
dojo.require("com.ibm.rdm.web.merge.ViewMergeWidget");
dojo.require("com.ibm.rdm.web.mergevvc.MergeState");
dojo.require("com.ibm.rdm.web.mergevvc.MergeStateStore");
dojo.require("com.ibm.rdm.web.mergevvc.MergeStateWriter");
dojo.require("com.ibm.rdm.web.mergevvc.RelatedArtifactsDeliveryManager");
dojo.require("com.ibm.rdm.web.util.vvcutils");
dojo.require("com.ibm.rdm.web.util.deferredutils");

dojo.requireLocalization("com.ibm.rdm.web.mergevvc", "VVCArtifactMergeWizardStep");

var	MessageHandler = com.ibm.rdm.web.common.MessageHandler,
	CompareConstants = com.ibm.rdm.web.compare.CompareConstants,
	ResourceType = CompareConstants.ResourceType,
	CompareLinksUtils = com.ibm.rdm.web.compare.compareLinksUtils,
	CompareUtils = com.ibm.rdm.web.compare.compareutils,
	InterimResultClient = com.ibm.rdm.web.comparevvc.InterimResultClient,
	VVCCompareConstants = com.ibm.rdm.web.comparevvc.VVCCompareConstants,
	VVCCompareUtils = com.ibm.rdm.web.comparevvc.vvccompareutils,
	MergeUtils = com.ibm.rdm.web.merge.mergeutils,
	MergeState = com.ibm.rdm.web.mergevvc.MergeState,
	MergeStateStore = com.ibm.rdm.web.mergevvc.MergeStateStore,
	MergeStateWriter = com.ibm.rdm.web.mergevvc.MergeStateWriter,
	RelatedArtifactsDeliveryManager = com.ibm.rdm.web.mergevvc.RelatedArtifactsDeliveryManager,
	VVCUtils = com.ibm.rdm.web.util.vvcutils,
	DeferredUtils = com.ibm.rdm.web.util.deferredutils,
	SELECTOR_AREA_WIDTH = 290,
	PANEL_SPACE_WIDTH = 8;
	


dojo.declare("com.ibm.rdm.web.mergevvc.VVCArtifactMergeWizardStep", [jazz.ui.wizard.WizardStep, dijit._Widget, dijit._Templated], {

	widgetsInTemplate: true,
	templatePath: dojo.moduleUrl("com.ibm.rdm.web", "mergevvc/templates/VVCArtifactMergeWizardStep.html"),
	
	// attach points
	// widget for display of errors/warnings
	_messageDisplayWidget: null,

	// The widget on which to display/select artifacts.
	_artifactSelectorWidget: null,
	
	// The widgets on which to display differences.
	_artifactMergeWidget: null,
	_viewMergeWidget: null,
	_tagMergeWidget: null,
	_messages: dojo.i18n.getLocalization("com.ibm.rdm.web.mergevvc", "VVCArtifactMergeWizardStep"),
	
	// private fields
	// URIs and names of the from and to configurations
	_targetConfigurationURI: null,
	_targetConfigurationName: null,
	_sourceConfigurationURI: null,
	_sourceConfigurationName: null,
	// The operation that we are currently displaying ('compare', 'accept', 'deliver')
	_operation: null,

	
	constructor: function() {
		this._headerText = this._messages["artifacts"]; //dojo.string.substitute(this._messages["artifacts"], ["JKE"]);
	},
	
	postCreate: function() {
		// In the event that the list of changes is empty, we only want to show one of the widgets.
		//   Hide all but the artifactMergeWidget.
		this._viewMergeWidget.hide();
		this._tagMergeWidget.hide();
		
		// Get the list of changes from VVC and display them
		try{
			this._artifactSelectorWidget._showLoading();
			this._artifactMergeWidget._showLoading();
			//console.log("VVCArtifactMergeWizardStep.postCreate()");
			InterimResultClient.loadInterimResult(VVCCompareConstants.COMPARE_ARTIFACT_NAMESPACE)
				.then(dojo.hitch(this, this._parseInterimResult))
				.then(dojo.hitch(this, function(params) {
					return CompareUtils.setCurrentProjectFromConfiguration(this._sourceConfigurationURI)
						.then(function() {
							// make sure we ultimately resolve with the parameters (interim results) 
							// as the subsequent chained functions expect to get them passed in. 
							return params; 
						});
				}))
				.then(dojo.hitch(this, this._setCurrentConfiguration))
				.then(dojo.hitch(this, function(resourceDeltas) {
					return CompareUtils.identifyModuleConstituents(this._targetConfigurationURI, this._sourceConfigurationURI, resourceDeltas);
				}))
				.then(dojo.hitch(this, function(resourceDeltas) {
					return CompareUtils.identifyRelatedArtifacts(this._targetConfigurationURI, this._sourceConfigurationURI, resourceDeltas);
				}))
				.then(dojo.hitch(this, function(resourceDeltas) {
					return CompareUtils.identifyCommentedArtifacts(this._targetConfigurationURI, this._sourceConfigurationURI, resourceDeltas);
				})) 
				.then(dojo.hitch(this, function(resourceDeltas){
                    return CompareLinksUtils.findLinkedArtifacts(this._targetConfigurationURI, this._sourceConfigurationURI, resourceDeltas);
                }))
				.then(dojo.hitch(this, this._initialiseDefaultMergeStates))
				.then(dojo.hitch(this, this._initialiseMergeStateManager))
				.then(dojo.hitch(this, this._showArtifactNames))
				.otherwise(VVCCompareUtils.checkForLockedArtifactsError);
		} catch(error) {
			MessageHandler.showMessageForError(error);
		}
	},
	
	resize: function(width, height) {
		// summary:
		//		size and position all the contained things to match the size we have been allocated.
		//		If width and height are not given, resize using the previous values.
		
		this._width = width = width || this._width;
		this._height = height = height || this._height;
		
		var selectorWidth = SELECTOR_AREA_WIDTH;
		var compareWidth = width - selectorWidth - PANEL_SPACE_WIDTH;
		var compareLeftPosition = selectorWidth + PANEL_SPACE_WIDTH;
		
		var messageHeight = this._messageDisplayWidget.domNode.offsetHeight;
		var contentHeight = height - messageHeight;

		// adjust widths and heights to account for the widgets being inside border nodes.
		// Numbers must match border sizes in CSS
		
		this._artifactSelectorWidget.resize(selectorWidth-2, contentHeight-2);	/* 1px border all round */
		this._artifactMergeWidget.resize(compareWidth-2, contentHeight-2);	/* 1px border all round */
		this._viewMergeWidget.resize(compareWidth-2, contentHeight-2);	/* 1px border all round */
		this._tagMergeWidget.resize(compareWidth-2, contentHeight-2);	/* 1px border all round */
		dojo.style(this._artifactCompareBorderNode, "left", compareLeftPosition+"px");
	},
	
	canFinish: function() {
		return true;
	},
	
	_onMessageResize: function() {
		// summary:
		//		Handler for event indicating that the error/warning/info message has resized
		//		(shown, hidden, expanded or collapsed).
		this.resize();
	},
	
	_parseInterimResult: function(interimResult) {
		// summary:
		//		Reads values from the interim results object, setting local variables as necessary. 
		//		Sets _sourceConfigurationURI, _sourceConfigurationName, _targetConfigurationURI, _targetConfigurationName 
		// returns:
		//		A hash of delta entries, keyed by concept URI. Each entry has a concept field, and
		//		may have a deltaTypes field.
		this._targetConfigurationURI = interimResult.targetConfigurationURI;
		this._targetConfigurationName = interimResult.targetConfigurationName;
		this._sourceConfigurationURI = interimResult.sourceConfigurationURI;
		this._sourceConfigurationName = interimResult.sourceConfigurationName;
		this._operation = interimResult.operation;
		// Pass on only the difference entries down the promise callback chain.
		return interimResult.entries[VVCCompareConstants.COMPARE_ARTIFACT_NAMESPACE];
	},
		
	_setCurrentConfiguration: function(param) {
		// summary:
		//		Sets the current configuration.  We use the to configuration as the default.  This will mean that it will be used for
		//		loading the type system and anything else used for comparison display, except in cases
		//		where we specifically override the configuration on a request.
		// returns:
		//		A promise that resolves with the supplied param once the configuration has been set.
		VVCUtils.setWorkingConfiguration(this._sourceConfigurationURI);
		return DeferredUtils.ensurePromise(param);
	},
	
	_initialiseDefaultMergeStates: function(resourceDeltaList) {
		// summary:
		//		Sets the merge state of artifacts in the merge state map, based on the changes
		//		to resources.  This effectively determines the default deliver/accept action
		//		for each resource.
		var interimResultURI = VVCUtils.getRequestParmValue('interimResultUri');
		interimResultURI = decodeURIComponent(interimResultURI);

		var stateWriter = new MergeStateWriter(interimResultURI);
		var stateStore = this._mergeStateStore = new MergeStateStore(stateWriter),
			concept, entry, mergeState, version, mergeStates;
						
		// initialise the merge state store with the current merge versions.  This sets its starting point to
		// match the VVC internal idea of the current merge state.

		for(concept in resourceDeltaList){
			entry = resourceDeltaList[concept];
			version = entry.mergeVersion;
			mergeState = null;
			if (VVCCompareUtils.hasAllDeltaTypes(entry, [CompareConstants.DELTA_TYPE_SOURCE_CHANGE, CompareConstants.DELTA_TYPE_TARGET_CHANGE])
				|| VVCCompareUtils.hasAllDeltaTypes(entry, [CompareConstants.DELTA_TYPE_SOURCE_DELETE, CompareConstants.DELTA_TYPE_TARGET_CHANGE])
				|| VVCCompareUtils.hasAllDeltaTypes(entry, [CompareConstants.DELTA_TYPE_SOURCE_CHANGE, CompareConstants.DELTA_TYPE_TARGET_DELETE])) {
				// A conflict - change in both source and target
				mergeState = MergeState.CONFLICT;
			} else if (VVCCompareUtils.hasDeltaType(entry, [CompareConstants.DELTA_TYPE_SOURCE_DELETE,
															CompareConstants.DELTA_TYPE_SOURCE_ADD,
															CompareConstants.DELTA_TYPE_SOURCE_CHANGE])) {
				// delete, add or change
				mergeState = MergeState.AUTO;
			}
			
			if (mergeState) {
				// Update the merge state store, but avoid sending notifications to VVC, since it
				// already has these versions in its merge map. 
				this._mergeStateStore.put(entry.concept, version, mergeState, true);
			}
		}
		
		// calculate our desired default merge position, and update the merge state store.
		mergeStates = [];
		for(concept in resourceDeltaList){
			entry = resourceDeltaList[concept];
			version = null;
			mergeState = null;
			if (VVCCompareUtils.hasDeltaType(entry, CompareConstants.DELTA_TYPE_VVC_FAKE_CONFLICT_INCORRECT_SOURCE)) {
                mergeState = MergeState.RESOLVED_NON_DEFAULT;
                version = entry.targetVersion;
                MessageHandler.showInfo(7812, null, null, null, false);
            } else if (VVCCompareUtils.hasDeltaType(entry, CompareConstants.DELTA_TYPE_VVC_FAKE_CONFLICT_INCORRECT_TARGET)) {
                mergeState = MergeState.CONFLICT;
                version = entry.sourceVersion;
                MessageHandler.showInfo(7812, null, null, null, false);
            } else if (VVCCompareUtils.hasAllDeltaTypes(entry, [CompareConstants.DELTA_TYPE_SOURCE_CHANGE, CompareConstants.DELTA_TYPE_TARGET_CHANGE])) {
				// A conflict - change in both source and target
				mergeState = MergeState.CONFLICT;
				version = entry.sourceVersion;
			} else if (VVCCompareUtils.hasAllDeltaTypes(entry, [CompareConstants.DELTA_TYPE_SOURCE_DELETE, CompareConstants.DELTA_TYPE_TARGET_CHANGE])) {
				// A conflict - delete in source change in target - no version indicating no version of this should be in the merged result.
				mergeState = MergeState.CONFLICT;
			} else if (VVCCompareUtils.hasAllDeltaTypes(entry, [CompareConstants.DELTA_TYPE_SOURCE_CHANGE, CompareConstants.DELTA_TYPE_TARGET_DELETE])) {
				// A conflict - change in source, delete in target
				mergeState = MergeState.CONFLICT;
				version = entry.sourceVersion;
			} else if (VVCCompareUtils.hasDeltaType(entry, [CompareConstants.DELTA_TYPE_SOURCE_ADD, CompareConstants.DELTA_TYPE_SOURCE_CHANGE])) {
				// add or change - version is the source.
				mergeState = MergeState.AUTO;
				version = entry.sourceVersion;
			} else if (VVCCompareUtils.hasDeltaType(entry, CompareConstants.DELTA_TYPE_SOURCE_DELETE)) {
				// delete - no version, indicating no version of this should be in the merged result.
				mergeState = MergeState.AUTO;
			}
			
			// If there are related artifacts that are in conflict, 'upgrade' the merge state to indicate a conflict, so that
			// it will get controls to deliver/not deliver to resolve the conflict.  Unless the concept has changed in its own
			// right it will not already be in the merge state store, and when it is added, the merge state store will give it
			// a flag to indicate that it is not to be merged itself, but by having it in the store its related resources
			// can be merged.
			if ((mergeState !== MergeState.CONFLICT) && MergeUtils.hasRelatedConflict(concept, resourceDeltaList)) {
				mergeState = MergeState.CONFLICT;
			}

			if (mergeState) {
				mergeStates.push({
					concept: entry.concept,
					resolveVersion: version,
					state: mergeState
				});
			}
		}
		
		return stateStore.putMulti(mergeStates).then(function() {
			// wait for this to complete since we do not want to 
			// to subscribe the RelatedArtifactsDeliveryManager to
			// merge state changes until we've finished initliasing.
			return resourceDeltaList;
		});

	},
	
	_initialiseMergeStateManager: function(resourceDeltaList) {
		// summary:
		//		Get the merge state menager intialised.  This sets it going waiting on
		//		events from the merge state store that trigger changes in the delivery
		//		state of related artifacts (bookmarks, links etc.) when an artifact is
		//		delivered.
		RelatedArtifactsDeliveryManager.init(this._mergeStateStore, resourceDeltaList);
		return resourceDeltaList;
	},
	
	_showArtifactNames: function(resourceDeltaList) {
		// summary:
		//		Tells the widget that displays the resource names, to display them
		this._artifactSelectorWidget.load(resourceDeltaList,
			 	this._targetConfigurationURI,
			 	this._sourceConfigurationURI,
			 	this._mergeStateStore,
			 	this._targetConfigurationName,
			 	this._sourceConfigurationName);

		// Keep a hash of the resource deltas for use when an artifact is selected.
		this._resourceDeltaCache = resourceDeltaList;
	},
	
	_onArtifactSelect: function(params) {
		// summary:
		//		Callback from the artifact selector widget when an artifact is selected.		
		try{
			// Get the list of all changed/added/removed resources from the selector widget.
			// TODO: Calculate these up here, and pass into the selector widget, rather than
			//		relying on selector widget building this list for us.  Would also be better
			//		if the resource deltas distinguished between link deltas and other change deltas.
			// Tell the artifact compare widget to display the artifact.
			var conceptURI = params.concept,
				resourceDelta = this._resourceDeltaCache[conceptURI];
			
			var mergeArgs = {
				displayType: params.displayType,
				targetURI: conceptURI,
				targetConfiguration: this._targetConfigurationURI,
				targetConfigurationName: this._targetConfigurationName,
				sourceURI: conceptURI,
				sourceConfiguration: this._sourceConfigurationURI,
				sourceConfigurationName: this._sourceConfigurationName,
				resourceDeltas: this._resourceDeltaCache,
				operation: this._operation,
				mergeStateStore: this._mergeStateStore
			};
			
			if (resourceDelta.type === ResourceType.VIEW) {
				this._artifactMergeWidget.hide();
				this._tagMergeWidget.hide();
				this._viewMergeWidget.show(mergeArgs)
				.otherwise(dojo.hitch(MessageHandler, MessageHandler.showMessageForError));
			} else if (resourceDelta.type === ResourceType.TAG) {
				this._artifactMergeWidget.hide();
				this._viewMergeWidget.hide();
				this._tagMergeWidget.show(mergeArgs)
				.otherwise(dojo.hitch(MessageHandler, MessageHandler.showMessageForError));
			} else {
				this._viewMergeWidget.hide();
				this._tagMergeWidget.hide();
				this._artifactMergeWidget.show(mergeArgs)
				.otherwise(dojo.hitch(MessageHandler, MessageHandler.showMessageForError));
			}
		} catch(error) {
			MessageHandler.showMessageForError(error);
		}
	}

});
})();
