/*
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2008, 2015.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 */
(function(){
dojo.provide("com.ibm.rdm.web.viewer.EmbeddedResourceLoader");

dojo.require("com.ibm.rdm.web.model.resources.ResourceManager");
dojo.require("com.ibm.rdm.web.common.MimeTypes");
dojo.require("com.ibm.rdm.web.common.MessageHandler");
dojo.require("com.ibm.rdm.web.review.ReviewCache");
dojo.require("com.ibm.rdm.web.server.client.RDMClient");
dojo.require("com.ibm.rdm.web.util.queryutils");
dojo.require("com.ibm.rdm.web.editor.EmbeddedResourceDecorator");
dojo.require("com.ibm.rdm.web.editor.EmbeddedCollectionViewer");
dojo.require("com.ibm.rdm.web.editor.EmbeddedDocumentViewer");
dojo.require("com.ibm.rdm.web.editor.EmbeddedDiagramImageViewer");
dojo.require("com.ibm.rdm.web.history.revisionsHelper");
dojo.require("com.ibm.rdm.web.ext.ArtifactExtension");
dojo.require("com.ibm.rdm.web.server.XhrUtils");
dojo.require("com.ibm.rdm.web.server.client.utils.WrapperResourceUtil");
dojo.require("com.ibm.rdm.web.editor.EditorConstants");
dojo.require("com.ibm.rdm.web.viewer.viewerutils");
dojo.require("com.ibm.rdm.web.common.controlTypes");
dojo.require("com.ibm.rdm.web.common.ResourceLinksHandler");
dojo.require("com.ibm.rdm.web.server.Headers");
dojo.require("com.ibm.rdm.web.util.vvcutils");
dojo.require("com.ibm.rdm.web.common.StatusDisplay");
dojo.require("com.ibm.rdm.web.notifications.ChangeNotifier");

dojo.requireLocalization("com.ibm.rdm.web.commenting", "resourceviewer");

/*
 * Util class for scanning DOM for and then loading embedded resources.
 * Largely refactored out of ResourceViewerWidget.
 * 
 * Basic usage is given a parent DOM node and some arguments
 * detailing reviewId, projectSnapshotURI and reviewParser it queries the DOM
 * looking for embedded resources.  At time of writing there is only one supported
 * that being a requirement. (Content type MimeTypes.REQUIREMENT_NEW).
 * These are represented in the saved XHTML as <a class="embedded href="resourceURI"/>
 * This class finds them, replaces them with a <div class="embedded"> and then loads
 * the resource and passes this off to an EmbeddedResourceDecorator along with the
 * parent <div>.
 * 
 *	NOTE:  As of 6.0, reviews are based on streams.  Therefore, it is no longer necessary to do
 *	special processing when rendering embedded artifacts that are in artifacts that are in a review.  
 *  The version of the embedded artifact that is in the stream will be used.
 */
	var ResourceManager = com.ibm.rdm.web.model.resources.ResourceManager;
	var EmbeddedResourceDecorator = com.ibm.rdm.web.editor.EmbeddedResourceDecorator; 
	var RDMClient = com.ibm.rdm.web.server.client.RDMClient;
	var queryutils = com.ibm.rdm.web.util.queryutils;
	var viewerutils = com.ibm.rdm.web.viewer.viewerutils;
	var revisionsHelper = com.ibm.rdm.web.history.revisionsHelper;
	var ArtifactExtension = com.ibm.rdm.web.ext.ArtifactExtension;
	var WrapperResourceUtil = com.ibm.rdm.web.server.client.utils.WrapperResourceUtil;
	var EditorConstants = com.ibm.rdm.web.editor.EditorConstants;
	var controlTypes = com.ibm.rdm.web.common.controlTypes;
	var ResourceLinksHandler = com.ibm.rdm.web.common.ResourceLinksHandler;
	var Headers = com.ibm.rdm.web.server.Headers;
	var VVCUtils = com.ibm.rdm.web.util.vvcutils;
	var StatusDisplay = com.ibm.rdm.web.common.StatusDisplay;
	var ChangeNotifier = com.ibm.rdm.web.notifications.ChangeNotifier;
	
	var messages = dojo.i18n.getLocalization(
				"com.ibm.rdm.web.commenting", "resourceviewer");
				
	var BATCH_SIZE = 5;

	var XML_ELEMENT_ATTRIBUTE_RDMTYPE = "rdmtype";

	
	com.ibm.rdm.web.viewer.EmbeddedResourceLoader = function (/*Element*/parentNode, /*Object*/args) {
		// args : {
		//  parentResourceURI: the top level artifact that contains the embedded resource
		//  reviewId: ,
		//  projectSnapshotURI: ,
		//  moduleBaselineURI: the baseline URI to use if we are showing within a module snapshot,
		//  displayDate: the date to use if we are showing within a module snapshot,
		//  reviewParser: ,
		//  remove: true|false - should the embedded resource be removable, also indicates edit mode
		//  onRemove: function - function to be called when the embedded resource widget is removed
		//  onCreate: function - a function called when the embedded resource widget is created.
		//  isRefresh: boolean - true if loading is being done for resource that is being refreshed
		// }
		// Summary: Create a new EmbeddedResourceLoader which will scan the supplied
		// node for embedded artifacts.
	   this.embeddedLevel = 0;
	   dojo.mixin(this, args);
	   
	   this._toDestroy = [];
	   this._connectHandles = [];
	   this._circularReferencesFound = false;
	   this.parentNode = parentNode;
	   
	};
	
	com.ibm.rdm.web.viewer.EmbeddedResourceLoader.prototype = {
		_toDestroy: null,
		_connectHandles: null,
		_circularReferencesFound: null,

		_loadEmbeddedImages: function(parentResource, embeddedLevel) {
			// Summary
			// Fetch the embedded images for the particular resource. Check for embedded images that have been deleted
			// or which cannot be viewed if the user does not have permission for the image's project and provide
			// an inline error information.

			var images = viewerutils.getEmbeddedImageNodes(parentResource);
			var loadDeferred = new dojo.Deferred();
			var maxImageCount = images.length;
			var imageCount = 0;
			var sself = this;

			// Chain the image fetch deferreds together.
			var fetchImage = dojo.hitch(this, function(i) {
				var imageNode = images[i];

				// When images are embedded in the primary text of a resource, we use an HTML 
				// <img> tag.  The src attribute of the URL points at the wrapped resource URL
				// minus any VVC configuration.  So, we cannot use the URI on the src attribute
				// to initially display the image, this is probably a design error.  So hide it
				// here and then reveal it once the image loads.
				
				dojo.style(imageNode, "display", "none");
				var loadedCon = dojo.connect(imageNode, "onload", this, function () {
					this.onImageLoad && this.onImageLoad(imageNode);

					dojo.style(imageNode, "display", "");
					// this should only happen the once
					dojo.disconnect(loadedCon);
					
					if (dojo.isIE) {
						// For IE, the image loading comes a bit later than other browsers. So defer the
						// event notification
						window.setTimeout(dojo.hitch(this, function(){
							ChangeNotifier.notifyListeners({
								eventTopic: com.ibm.rdm.web.notifications.EventTopics.RESET_GOTO_ITEM_AT_IMG_LOADED
							});
						}), 400);
					}
					else {
						ChangeNotifier.notifyListeners({
							eventTopic: com.ibm.rdm.web.notifications.EventTopics.RESET_GOTO_ITEM_AT_IMG_LOADED
						});
					}
				});

				var imageSrc = imageNode.attributes.getNamedItem("src").value;
				
				var currentGlobalConfig = VVCUtils.getGlobalConfigurationUri(),
					currentConfiguration = VVCUtils.getCurrentConfigurationUri();
				if (currentGlobalConfig) {
					imageSrc += "&oslc.configuration="+encodeURIComponent(currentGlobalConfig);
				} else {
					imageSrc += "&vvc.configuration="+encodeURIComponent(currentConfiguration);
				}
				
				if (WrapperResourceUtil.isWrapperResourceURI(imageSrc)) {
					var parmIndex = imageSrc.lastIndexOf('?');
					var baseURL = imageSrc.substring(0, parmIndex);

					// Determine if the etag based around a snapshot must be realized.
					var snapshotQueryDeferred = null;
					// CAN THIS CODE BE REMOVED ////////////////////
					if (this.projectSnapshotURI && baseURL.length > 0) {
						// If in the scope of a snapshot, query for proper etag, then modify imageSrc URL to include the revision information
						snapshotQueryDeferred = com.ibm.rdm.web.model.resources.ResourceManager.getResourceDescribe({
							resourceURI: baseURL,
							projectSnapshotURI: this.projectSnapshotURI
						});
						snapshotQueryDeferred.addCallback(this, function(snapQueryResponse) {
							var revisionQueryParamIndex = snapQueryResponse.resourceLocation.indexOf("?") + 1;
							imageSrc += "&" + snapQueryResponse.resourceLocation.substring(revisionQueryParamIndex);
							return snapQueryResponse;
						});
					}
					else {
						// Otherwise just immediately callback
						snapshotQueryDeferred = new dojo.Deferred();
						snapshotQueryDeferred.callback();
					}

					snapshotQueryDeferred.addCallback(this, function(snapQueryResponse) {
						var onImageLoadError = dojo.hitch(this, function(){
							if (imageSrc.length > 0) {
								var deferred = RDMClient.doHead({
									uri: imageSrc,
									skipErrors: true,
									projectSnapshotURI: this.projectSnapshotURI
								});
								deferred.addErrback(this, function(response) {
									if (sself.remove && (embeddedLevel === 0)) {
										// if remove set, we are in edit mode, display the
										// error differently so the invalid image can be removed
										dojo.removeClass(imageNode, EditorConstants.IMG_EMBED_ERROR_CLASS);
										dojo.addClass(imageNode, EditorConstants.IMG_EMBED_EDIT_ERROR_CLASS);
										dojo.style(imageNode, "display", ""); // display the invalid image indicator
									}
									else {
										var errorDiv = document.createElement("div");
										var errorText = null;
										if (response.status === 403) {
											errorText = document.createTextNode(messages.imageNotAuthorized);
										}
										else if (response.status === 410) {
											errorText = document.createTextNode(dojo.string.substitute(messages.imageNotFound,
												[ imageSrc ]));
										}
										else {
											var errorMsg = dojo.string.substitute(messages.imageError, [ imageSrc,
													response.status ]);
											errorText = document.createTextNode(errorMsg);
										}
										errorDiv.appendChild(errorText);
										dojo.addClass(errorDiv, EditorConstants.IMG_EMBED_ERROR_CLASS);
										dojo.place(errorDiv, imageNode, "before");
										imageNode.style.display = "none";
									}
									imageCount = imageCount + 1;
									if (imageCount < maxImageCount) {
										fetchImage(imageCount);
									}
									else {
										loadDeferred.callback(true);
									}
									return response;
								});
							}
						});
						var setImageSrcAttribute = dojo.hitch(this, function(){
							if (this.displayDate) {
								// For module snapshots, we must find the appropriate dated result
								var wrapperDeferred = WrapperResourceUtil.getWrapperResourceURI(baseURL);
								wrapperDeferred.addCallback(this, function(wrapperResponse) {
									// handle the case where the embedded image is not part of the module baseline, calculate
									// the revision to use in this case
									var deferred = revisionsHelper.getNewestPastRevisionByDate(imageSrc,
										this.displayDate);
									deferred.addCallback(this, function(revisionsInfo) {
										imageNode.src = imageSrc += ("&revision=" + revisionsInfo.revisionToUse);
									});

									return wrapperResponse;
								});
							}
							// If in a snapshot, imageSrc will already have proper revision from the snapshot query above
							else if (sself.projectSnapshotURI) {
								imageNode.src = imageSrc;
							}
							// We use preventCache even though we're looking at the most current version due to image
							// caching that the browsers do. If an image is changed in a session, preventCache needs
							// to be used to ensure the most recent version of the image is displayed.
							// Note: we're using '(+(new Date()))' here, just like dojo does in dojo/request/util.js.
							else {
								imageNode.src = imageSrc + "&preventCache=" + (+(new Date()));
							}

							// If an error occurs when loading imageNode.src, run custom code to deal with it.
							imageNode.onerror = onImageLoadError;

							imageCount = imageCount + 1;
							if (imageCount < maxImageCount) {
								fetchImage(imageCount);
							}
							else {
								loadDeferred.callback(true);
							}
						});
						setImageSrcAttribute();
						return snapQueryResponse;
					});
				}
				else {
					imageCount = imageCount + 1;
					if (imageCount < maxImageCount) {
						fetchImage(imageCount);
					}
					else {
						loadDeferred.callback(true);
					}
				}
			});

			// start the chained fetching deferreds
			fetchImage(imageCount);
			return loadDeferred;
		},

		_loadEmbeddedArtifacts: function(/*Element?*/parentResource, embeddedLevel, reviewParser) {
			parentResource = parentResource || this.parentNode;
			embeddedLevel = embeddedLevel || this.embeddedLevel;
			var richTextArtifacts = dojo.query(EditorConstants.RICH_TEXT_EMBEDS, parentResource);
			var curIdx = 0;
			var loadDeferred = new dojo.Deferred();
			
			var embedCount = richTextArtifacts.length;
			
			var loadEmbeds = dojo.hitch(this, function(/*Object?*/reviewParser) {
				
				if (richTextArtifacts.length === 0) {
					loadDeferred.callback(true);
				}
				
				var load = dojo.hitch(this, function (rtNodes) {
					// Summary: This function does the actual loading and begins the
					// process of expanding out the <a> tags into the embedded resource
					// representation
					var deferreds = [];
					
					dojo.forEach(rtNodes, function (rtNode) {
						var linkHref = rtNode.attributes.getNamedItem("href").value;
						if (dojo.isFF) {
							// workaround 32361, which is a FF bug that should be fixed 
							// embedding done via paste operation, FF changing absolute URIs to relative on
							// paste
							if (linkHref.indexOf("http") !== 0) {				
								var url = new dojo._Url(com.ibm.rdm.web.FRONTING_SERVICE_RESOURCES, linkHref);
								linkHref = url.toString();
								rtNode.setAttribute("href", linkHref);
							}
						}
					
						var id  = dojo.attr(rtNode, "id");
						var rtParent = rtNode.parentNode;
						var embeddedContainer = rtParent.ownerDocument.createElement("div");
				
						// Put a "Loading..." message in the div until the artifact is loaded
						var fetchingSpan = rtParent.ownerDocument.createElement("span");
						var img = rtParent.ownerDocument.createElement("img");
						img.src = dojo.moduleUrl("com.ibm.rdm.web", "common/images/loading.gif");
						dojo.addClass(img, "embeddedLoadElement");
						fetchingSpan.appendChild(img);
						var labelSpan = rtParent.ownerDocument.createElement("span");
						labelSpan.innerHTML = messages.loadEmbed;
						dojo.addClass(labelSpan, "embeddedLoadElement");
						fetchingSpan.appendChild(labelSpan);
						dojo.addClass(fetchingSpan, "embeddedLoadMessage");
						embeddedContainer.appendChild(fetchingSpan);
	
						rtParent.insertBefore(embeddedContainer, rtNode.nextSibling);
						embeddedContainer.originalAnchor = rtNode;
		  
						var deferred = this._fetchEmbeddedResource(linkHref, embeddedContainer, com.ibm.rdm.web.common.MimeTypes.CONTENT_TYPE_RICH_TEXT, embeddedLevel, reviewParser);
						deferred.addCallback(this, function(response) {
							embedCount = embedCount - 1;
							if (embedCount == 0) {
								loadDeferred.callback(response);
							}
							return response;
						});
						
						deferreds.push(deferred);
						
					}, this);
					
					return new dojo.DeferredList(deferreds);
					
				});
				
				var loadNextBatch = function () {
					// Summary:
					//		This function will load the next batch of embedded resources.
					
					var batchSize = BATCH_SIZE,
						toLoad; // the segment of the total embedded resources we are going to load, a sub section of richTextArtifacts
								// in the parent context
								
					// figure out whether by splitting up the array by batch size we will go off the end....
					if (curIdx + batchSize >= richTextArtifacts.length) {				
						batchSize = richTextArtifacts.length - curIdx;
					} 
					// show a helpful message so the user can see what is going on.
					if (embeddedLevel === 0) {
						if (richTextArtifacts.length === 1) {
							// when inserting multiple artifacts, these are done individually, so don't use a numbered
							// message for this case since this is confusing when doing the loading for inserting 
							// multiple artifacts
							StatusDisplay.showStatusMessage(dojo.string.substitute(messages["loadEmbed"]));
						} else {
							StatusDisplay.showStatusMessage(dojo.string.substitute(messages["loadingEmbeds"], [curIdx + batchSize, richTextArtifacts.length]));
						}
					} 
					
					toLoad = richTextArtifacts.slice(curIdx, curIdx+batchSize);
					if (curIdx < richTextArtifacts.length) {
						// load the next batch and then after a pause so we can update the display, load the next
						// batch.
						load(toLoad).addCallback(function(){
							setTimeout(loadNextBatch);
						});
						curIdx += batchSize;
					} else {
						if (embeddedLevel === 0) {
							StatusDisplay.hideStatusMessage();
						}
					}
				};
				
				setTimeout(loadNextBatch);
			});

			if (reviewParser) {
				loadEmbeds(reviewParser);
			}
			else if (this.reviewId) {
				var onReviewLoaded = function(response) {
					loadEmbeds(response.getReviewParser());
					return response;
				};
				var reviewDeferred = com.ibm.rdm.web.review.ReviewCache.getReview(this.reviewId);
				reviewDeferred.addCallback(onReviewLoaded);
			}
			else {
				loadEmbeds();
			}
			return loadDeferred;
		},
	  
		loadEmbeddedResources: function(/*Element?*/parentResource, embeddedLevel, reviewParser) {
			parentResource = parentResource || this.parentNode;
			embeddedLevel = embeddedLevel || 0;
			var images = viewerutils.getEmbeddedImageNodes(parentResource);
			var richTextArtifacts = dojo.query(EditorConstants.RICH_TEXT_EMBEDS, parentResource);
			if (images.length === 0 && (richTextArtifacts.length === 0)) {
				return null;
			}

			var deferred;
			if (images.length > 0 && (richTextArtifacts.length === 0)) {
				deferred = this._loadEmbeddedImages(parentResource, embeddedLevel);
			}
			else if (images.length > 0 && (richTextArtifacts.length > 0)) {
				this._loadEmbeddedImages(parentResource, embeddedLevel);
				deferred = this._loadEmbeddedArtifacts(parentResource, embeddedLevel, reviewParser);
			}
			else {
				deferred = this._loadEmbeddedArtifacts(parentResource, embeddedLevel, reviewParser);
			}
			return deferred;
		},
	  

		_fetchEmbeddedResource: function(resourceURI, node, contentType, embeddedLevel, reviewParser) {
			// summary:
			//		Load an embedded resource into the currently opened rich-text
			//		based artifact.
			resourceURI = this._trimResourceSrcAttribute(resourceURI);
	
			var fetchDeferred = new dojo.Deferred();
			var onGetSuccess = function(response) {
				if (!(response instanceof Error) && !response.err) {
					// success
					var self = this;
					var isCircular = false;
					var resourceType = response.getEntityType() ;
					var resourceTypeIndex = resourceType.lastIndexOf('#');
					resourceType = resourceType.substring(resourceTypeIndex + 1);
					if(resourceType === "WrapperResource"){
						var widget = new com.ibm.rdm.web.editor.EmbeddedDocumentViewer(node, response, embeddedLevel,{
							remove: this.remove,
							onCreate: this.onCreate,
							onRemove: this.onRemove
						}, this.projectURI, reviewParser, this.displayDate ? null : this.projectSnapshotURI);
						
						this._toDestroy.push(widget);
					}
					else{
						var realContentType = response.getMimeType();
						realContentType = queryutils.getNormalizedContentType(realContentType);
						var formatTypeIndex = realContentType.lastIndexOf('#');
						var formatType = realContentType.substring(formatTypeIndex + 1);
						if (formatType === "Text") {
							// check for a circular reference, if found do not embed the artifact, instead provide 
							// an error message
							if (this._isCircularEmbed(resourceURI, node, embeddedLevel)) {
								var errorText = dojo.string.substitute(messages.embeddedArtifactCircular, [response.getID(),response.getTitle()]);
								this._showEmbeddedResourceError(node, embeddedLevel, errorText);
								this._circularReferencesFound = true;
								isCircular = true;
							} else {
								var widget = new EmbeddedResourceDecorator(node, response, embeddedLevel, {
									remove: this.remove,
									onCreate: this.onCreate,
									onRemove: this.onRemove
								}, this.projectURI, reviewParser);
								this._toDestroy.push(widget);
								if (embeddedLevel >= 1 && widget.isMinimized()) {
									// stop processing since only top level minimized
									// embeds can be expanded
								} else {
									var loader = this.loadEmbeddedResources(widget.contents,/*node,*/embeddedLevel + 1, reviewParser);
									if (loader) {
										loader.then(
											dojo.hitch(this, function(loadResponse) {
												fetchDeferred.callback(true);
												return loadResponse;
											})); //tail recursive call
										return response;
									} 
								}
							}
						}
						else if (formatType === "Collection") {
		
							var widget = new com.ibm.rdm.web.editor.EmbeddedCollectionViewer(node, response, embeddedLevel,
								{
									remove: this.remove,
									onCreate: this.onCreate,
									onRemove: this.onRemove
								}, this.projectURI, reviewParser, this.displayDate ? null : this.projectSnapshotURI);
							this._toDestroy.push(widget);
						} else if (formatType === "Diagram") {
							var widget = new com.ibm.rdm.web.editor.EmbeddedDiagramImageViewer(node, response, embeddedLevel,
									{
										remove: this.remove,
										onCreate: this.onCreate,
										onRemove: this.onRemove
									}, this.projectURI, reviewParser, this.displayDate ? null : this.projectSnapshotURI);
								this._toDestroy.push(widget);
						}
						/* [SW] TODO: remove this switch statement and refactor invocation of ext artifact viewer
						 * into a simple else statement
						 */
						var widget = null;
						switch (realContentType) {
						case com.ibm.rdm.web.common.MimeTypes.JPG.contentType:
						case com.ibm.rdm.web.common.MimeTypes.JPE.contentType:
						case com.ibm.rdm.web.common.MimeTypes.BMP.contentType:
						case com.ibm.rdm.web.common.MimeTypes.JPEG.contentType:
						case com.ibm.rdm.web.common.MimeTypes.GIF.contentType:
						case com.ibm.rdm.web.common.MimeTypes.PNG.contentType:
						case com.ibm.rdm.web.common.MimeTypes.ICO.contentType:
							var nodeSrc =
								node.attributes.getNamedItem("imgSrc").value;
							if(this.projectSnapshotURI) {
								nodeSrc = com.ibm.rdm.web.server.XhrUtils
									.getRevision(response);
							}
							nodeSrc += "&accept=none&private";
							node.src = nodeSrc;
							dojo.removeClass(node, "dijitDisplayNone");
						break;
		
						case com.ibm.rdm.web.common.MimeTypes.CONTENT_TYPE_STORY_BOARD:
		
							var normalizedResourceURI = resourceURI;
							widget = com.ibm.rdm.web.viewer.embeddedResourceFactory.create(this.projectURI,
								normalizedResourceURI, response, node, this.reviewParser, this.projectSnapshotURI);
							node = widget.domNode;
							// Note: To be in alignment with the rich client, when the user clicks on an
							// embedded diagram (e.g. a Sketch, Part, etc) he/she is actually selecting
							// the _paragraph_ that contains the diagram (and text may appear above/below
							// the diagram within this paragraph).  For this reason, we take care here to
							// set an appropriate attribute on the parent node (a 'p' tag) so that when
							// it's selected the Commenting sidebar area is able to say something like
							// "Create a new comment for: Paragraph".
							node.parentNode.setAttribute(XML_ELEMENT_ATTRIBUTE_RDMTYPE,
								controlTypes.CONTROL_TYPES.Paragraph);
							// Add ability to double-click through to the embedded artifact.
							//var shortnameResourceURI = queryutils.getLastPathSegment(resourceURI);
							if (embeddedLevel === 0) {
								self._connectHandles.push(self.connect(node, "ondblclick", function(event) {
									self._handleArtifactDoubleClicked(event, resourceURI);
								}));
							}
						break;
						case com.ibm.rdm.web.common.MimeTypes.CONTENT_TYPE_COLLECTION:
							var normalizedResourceURI = resourceURI;
							var span = document.createElement("SPAN");
							node.innerHTML = "";
							node.appendChild(span);
							widget = com.ibm.rdm.web.viewer.embeddedResourceFactory.create(this.projectURI,
								normalizedResourceURI, response, span, null, this.projectSnapshotURI);
							node = widget.domNode;
						break;
						default:
							//TODO: Need Project context!
							if (ArtifactExtension.isExtendedType(realContentType, response.projectURI)) {
								// If this is being called from a module, the snapshotUri needs to bleed through to the converter
								if (!response.baselineURI && this.moduleBaselineURI) {
									response.baselineURI = this.moduleBaselineURI;
								}
								var handler = ArtifactExtension.getArtifactHandler(realContentType, response.projectURI);
								widget = handler.getEmbeddedResourceViewer({
									node: node,
									response: response,
									embeddedLevel: embeddedLevel,
									style: null,
									options: {
										remove: this.remove,
										onCreate: this.onCreate,
										onRemove: this.onRemove
									},
									projectURI: response.projectURI,
									reviewParser: reviewParser,
									projectSnapshotURI: response.baselineURI,
									resourceURI: response.resourceURI
								});
								break;
							}
						}
						if(widget) {
							this._toDestroy.push(widget);
						}
					}
					if (isCircular) {
						fetchDeferred.callback(false);
					} else {
						fetchDeferred.callback(true);
					}
				}
				else {
					// do a HEAD to figure out what the issue is, ResourceManager.get does not return
					// what error occurred
					var self = this;
					var handleError = function(headResponse) {
						var errorText = null;
						if (headResponse.status === 403) {
							errorText = messages.embeddedArtifactNotAuthorized;
						}
						else if (headResponse.status === 410) {
							errorText = dojo.string.substitute(messages.embeddedArtifactNotFound, [ resourceURI ]);
						}
						else {
							if (response instanceof Error) {
								errorText = dojo.string.substitute(messages.embeddedArtifactError, [ resourceURI,
										response.toString() ]);
							}
							else {
								errorText = dojo.string.substitute(messages.embeddedArtifactError, [ resourceURI,
										response.err ]);
							}
						}
						self._showEmbeddedResourceError(node, embeddedLevel, errorText);
						fetchDeferred.callback(false);
					};
	
					var deferred = RDMClient.doHead({
						uri: resourceURI,
						skipErrors: true
					});
					deferred.addCallback(this, function(response) {
						handleError(response);
						return response;
					});
					deferred.addErrback(this, function(response) {
						handleError(response);
						return response;
					});
				}
				return response;
			};
	
			if (this.displayDate) {
				//We have a displayDate - must be in a module snapshot, and embedded resources should be displayed as at this date.
				var self = this;
				var deferred = RDMClient.doHead({
					uri: resourceURI,
					skipErrors: true
				});
				deferred.addCallback(this, function(response) {
					var resourceDescriptionDeferred = com.ibm.rdm.web.model.resources.ResourceManager
						.getResourceDescribe({
							resourceURI: resourceURI
						});
					resourceDescriptionDeferred.addCallback(self, function(resourceDescription) {
						if (resourceDescription.boundArtifact) {
							var onNewestPastRevisionByDateComplete = function(revisionsInfo) {
								var revisionToUse = revisionsInfo.revisionToUse;
								ResourceManager.get({
									resourceURI: resourceDescription.boundArtifact,
									revision: revisionToUse,
									reportErrors: true
								}, dojo.hitch(self, onGetSuccess));
							};
							var deferred = revisionsHelper.getNewestPastRevisionByDate(resourceDescription.boundArtifact, self.displayDate);
							deferred.addCallback(onNewestPastRevisionByDateComplete);
							return response;
						}
						else {
							var onNewestPastRevisionByDateComplete = function(revisionsInfo) {
								var revisionToUse = revisionsInfo.revisionToUse;
								ResourceManager.get({
									resourceURI: resourceURI,
									revision: revisionToUse,
									reportErrors: true
								}, dojo.hitch(self, onGetSuccess));
							};
							var deferred = revisionsHelper.getNewestPastRevisionByDate(resourceURI, self.displayDate);
							deferred.addCallback(onNewestPastRevisionByDateComplete);
							return response;
						}
					});
				});
				deferred.addErrback(this, function(response) {
					onGetSuccess(response);
					return response;
				});
			}
			else if (this.projectSnapshotURI) {
				var self = this;
				var deferred = RDMClient.doHead({
					uri: resourceURI,
					skipErrors: true
				});
	
				deferred.addCallback(this, function(response) {
					//we need to run a query to get the exact revion of an artifact in snapshot
					var resourceDescriptionInSnapshotDeferred = com.ibm.rdm.web.model.resources.ResourceManager
						.getResourceDescribe({
							resourceURI: resourceURI,
							projectSnapshotURI: this.projectSnapshotURI
						});
					resourceDescriptionInSnapshotDeferred.addCallback(self, function(resourceDescriptionInSnapshot) {
						var resourceInSnapshotRevision =
							ResourceLinksHandler.createRevisionURIFromResourceLocationInBaseline(
								resourceURI,
								resourceDescriptionInSnapshot.resourceLocation
							);

						if (resourceDescriptionInSnapshot.boundArtifact) {
							var boundArtifactDeferred = ResourceManager.getResourceDescribe({
								resourceURI: resourceDescriptionInSnapshot.boundArtifact,
								projectSnapshotURI: this.projectSnapshotURI,
								reportErrors: true
							});
							boundArtifactDeferred.addBoth(dojo.hitch(self, function(results) {
								ResourceManager.get({
									resourceURI: resourceURI,
									contentTypeOverride: results.contentType,
									revision: resourceInSnapshotRevision,
									projectSnapshotURI: this.projectSnapshotURI,
									reportErrors: true
								}, dojo.hitch(self, onGetSuccess));
							}));
							return boundArtifactDeferred;
						}
						else {
							ResourceManager.get({
								resourceURI: resourceURI,
								revision: resourceInSnapshotRevision,
								projectSnapshotURI: this.projectSnapshotURI,
								reportErrors: true
							}, dojo.hitch(self, onGetSuccess));
	
							return resourceDescriptionInSnapshot;
						}
					});
					resourceDescriptionInSnapshotDeferred.addErrback(self, function(error) {
						// if the resource is not found in the snapshot, do a regular get, this
						// could be the case when an embedded artifact is from another project
						ResourceManager.get({
							resourceURI: resourceURI,
							reportErrors: true
						}, dojo.hitch(self, onGetSuccess));
						return error;
					});
					return response;
				});
				deferred.addErrback(this, function(response) {
					onGetSuccess(response);
					return response;
				});
			}
			else {
				if (this.isRefresh) {
					this._getRefreshedResource(resourceURI).then(
						dojo.hitch(this, onGetSuccess),
						dojo.hitch(this, onGetSuccess)
					);
				} else {
					ResourceManager.get({
						resourceURI: resourceURI,
						reportErrors: true
					}, dojo.hitch(this, onGetSuccess));
				}
			}
			return fetchDeferred;
		},
		
		_showEmbeddedResourceError: function(node, embeddedLevel, errorText) {
			// If we're in editMode (flagged via remove being true), render the
			// error differently so user can remove the invalid embed if necessary
			if (this.remove) {
				var widget = new EmbeddedResourceDecorator(node, null, embeddedLevel, {
					remove: true,
					onRemove: this.onRemove,
					errorContents: '<p>' + errorText + '</p>'
				});
				this._toDestroy.push(widget);
			}
			else {
				var messageDiv = dojo.doc.createElement('div');
				messageDiv.innerHTML = '<p>' + errorText + '</p>';
				dojo.addClass(messageDiv, EditorConstants.ARTIFACT_EMBED_ERROR_CLASS);
				// remove loading... message
				node.removeChild(node.firstChild);
				node.appendChild(messageDiv);
			}
		},
		
		_isCircularEmbed: function(resourceURI, node, embeddedLevel) {
			// summary:
			// 		Checks to see if the resource that is about to be embedded is already referenced in a parent embedded resource.  If so,
			//		this creates a circular reference, so we should stop processing.
			//	
			if (embeddedLevel === 0) {
				return resourceURI === this.parentResourceURI;
			}
			var parentNode = node.parentNode;
			var currentLevel = embeddedLevel;
			while (parentNode && (currentLevel >= 0)) {
				if ((currentLevel === 0) && (resourceURI === this.parentResourceURI)) {
					return true;
				} else {
					if (dojo.hasClass(parentNode, EditorConstants.RICH_TEXT_EMBED_WRAPPER_CLASS)) {
						if (currentLevel !== embeddedLevel) {
							var child = parentNode.firstChild;
							if (child && (child.tagName == "A")) {
								var href = child.getAttribute("href");
								if (href === resourceURI) {
									return true;
								}
							}
						}
						currentLevel = currentLevel - 1;
					}
				}
				parentNode = parentNode.parentNode;
			}
			return false;
		},
		
		circularReferencesFound: function() {
			// summary:
			// 		Returns whether or not circular references were found when loading the embedded resource for the
			//		given parentResourceURI.
			return this._circularReferencesFound;
		},

		_getRefreshedResource: function(resourceURI) {
			// summary:
			// 		Gets the current version of the embedded resource.
			//		Introduced for work item 53544: Refresh action on page does not refresh contents of embedded artifacts
			
			var refreshedResource = new dojo.Deferred();
			// If we are loading embedded resources due to a refresh action, use the cached version of the embedded
			// artifact to get the etag and see if it is the lastest version.
			ResourceManager.get({
				resourceURI: resourceURI,
				reportErrors: true
			}).then( 
				dojo.hitch(this, function(response){
					// pass along IF_NONE_MATCH arg so that we don't retrieve the resource data
					// if it is the most current version
		    		var headers = {};
		    		headers[Headers.IF_NONE_MATCH] = response.etag;
    				var deferred = ResourceManager.get({
						resourceURI: resourceURI,
						reportErrors: true,
						skipCache: true,
						xhrArgs: { headers: headers }
					});
					deferred.addCallback(dojo.hitch(this, function(latest) {
						refreshedResource.callback(latest);
						return latest;
					}));
					deferred.addErrback(dojo.hitch(this, function(error) {
						refreshedResource.errback(error);
						return error;
					}));
					return response;
				}),
				dojo.hitch(this, function(error) {
					refreshedResource.errback(error);
					return error;
				})
			);
			return refreshedResource;
		},
		
		destroy: function() {
			if(!this._beingDestroyed) {
				this._beingDestroyed = true;
				for(var idx in this._toDestroy) {
					var doomed = this._toDestroy[idx];
					if(doomed) {
						if(dojo.isFunction(doomed.destroyRecursive)) {
							doomed.destroyRecursive();
						}
						else if(dojo.isFunction(doomed.destroy)) {
							doomed.destroy();
						}
						doomed = null;
						this._toDestroy[idx] = null;
					}
				}
				
				for(var idx in this._connectHandles) {
					var doomedHandle = this._connectHandles[idx];
					if(doomedHandle) {
						try { dojo.disconnect(doomedHandle); } catch(e){}
					}
					doomedHandle = null;
					this._connectHandles[idx] = null;
				}
			}
		},
		
		destroyRecursive: function() {
			return this.destroy();
		},

		_checkExternallyEmbedded: function(node, resourceURI /* String, src attribute */){
			try {
				if (com.ibm.rdm.web.links.linkutils.isFromAnotherRepository(resourceURI)) {
					var msg = dojo.string.substitute(messages.embeddeddifferentrepo, [resourceURI]);
					var errorSpan = document.createElement("SPAN");
					var textNode = document.createTextNode(msg);
					errorSpan.appendChild(textNode);
					dojo.addClass(errorSpan, "error");
					if (node.tagName === "img" || node.tagName === "IMG") {
						dojo.place(errorSpan, node, "after");
						node.parentNode.removeChild(node);
					}
					else {
						node.innerHTML = "";
						node.appendChild(errorSpan);
					}
					return true;
				}
			}
			catch (e) {
				com.ibm.rdm.web.common.MessageHandler.showError(7204, [resourceURI], e);
			}
			return false;
		},
		
		_trimResourceSrcAttribute: function(resourceSrc){
			if (resourceSrc.indexOf("#") >= 0) {
				resourceSrc = resourceSrc.substring(0, resourceSrc.indexOf("#"));
			}
			return resourceSrc;
		}
	};
	

})();
