/**********************************************************************************************************************************
 * Licensed Materials - Property of IBM
 * Restricted Materials of IBM.  Reproduction, modification and redistribution are prohibited.
 * Task-Oriented Quick Tracking and Planning feature for Rational Team Concert and Jazzhub.
 * (c) Copyright IBM Corporation 2014, 2015.
 * U.S. Government Users Restricted Rights:  Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp. 
 **********************************************************************************************************************************/

define(['framework/App',
        'areas/iterations/models/IterationCollection',
        'framework/common/models/Action',
        'framework/common/models/ActionCollection',
        'areas/iterations/models/IterationConstants',
        'areas/common/models/CommonConstants',
        'areas/workitems/models/WorkItemConstants',
        'areas/filters/models/FilterConstants',
        "areas/projects/models/ProjectConstants",
        "areas/servers/models/ServerConstants",
        'areas/workitems/models/SprintPlanTreeSection',
		'areas/workitems/models/SectionCollection',
        'areas/workitems/controllers/AbstractWorkItemCollectionController',
		'areas/workitems/controllers/AbstractPlanningPageController',
        'areas/workitems/controllers/utils/IterationsProvider',
        'areas/workitems/views/SprintPlanningContainerView',
    	'areas/workitems/controllers/utils/TreeViewParentChildModelUpdater'], 
function(App,
		IterationCollection,
		Action, 
		ActionCollection,
		IterationConstants, 
		CommonConstants, 
		Constants, 
		FilterConstants, 
		ProjectConstants, 
		ServerConstants,
		SprintPlanTreeSection, 
		SectionCollection,
		AbstractWorkItemCollectionController, 
		AbstractPlanningPageController,
		IterationsProvider,
		SprintPlanningContainerView, 
		TreeViewParentChildModelUpdater) {
	
	return AbstractPlanningPageController.extend({
		
		DEFAULT_COLUMNS: AbstractPlanningPageController.prototype.DEFAULT_COLUMNS.concat([Constants.propResolution]),
		
		_parentChildModelUpdater: new TreeViewParentChildModelUpdater(),
		
		// -- Move following to constructor ---
		
		canDisplayAsTable: function() {
			return false;
		},
		
		canDisplayAsList: function() {
			return false;
		},
		
		canDisplayAsLanes: function() {
			return true;
		},
		
		getLaneType: function() {
			return Constants.viewTypeListLanes;
		},
		
		shouldDisplaySprintFilterInNavbar: function() {
			return true; //App.isJazzhub;
		},
		
		shouldDisplayEditSprintsInNavbar: function(context){
			if (!App.isSmallScreen && context.project.get(ProjectConstants.isCurrentUserProjectAdmin)) {
				return true;
			}
			return false;
		},
		
		showEditor: function(viewTypeHint) {
            AbstractWorkItemCollectionController.prototype.showEditor.call(this, viewTypeHint);
            
            var preferences = App.preferences.at(0);
            
            this.sectionActions = new ActionCollection();
            
            // This is inverted.  Need to start in the abstract section and work up!
            var addAction = _.bind(function(label, excludeResolved) {
            	var action = new Action({
                    label: label,
                    isSelectable: true,
                    excludeResolved: excludeResolved,
                    handler: _.bind(function handleSelectionChanged() {
                    	if (this._currentEditor) {
	                        this._currentEditor.sections.each(function(section) {
	                            section.set(FilterConstants.excludeResolved, excludeResolved);
	                        });
                            preferences.set(FilterConstants.excludeResolvedPreferencesKey, excludeResolved);
                            preferences.save();
	                        this.refresh();
	                    }
                    }, this)
                });
                this.sectionActions.add(action);
            }, this);
            
            addAction(App.messages.SprintPlanningController_unresolvedWorkItems, true);
            addAction(App.messages.SprintPlanningController_allWorkItems, false);

            App.layout.showSectionActions(this.sectionActions);
            
            var preference = preferences.get(FilterConstants.excludeResolvedPreferencesKey);
            if(preference || preference === undefined){
        		this.sectionActions.first().select();
            } else {
            	this.sectionActions.last().select();
            }
        },
		
		createOptions: function(hints, viewType) {
			var options = AbstractWorkItemCollectionController.prototype.createOptions.call(this, _.extend(hints, {
				doNotTrimReleaseIterationForJazzhub: true,
				dontHideEditActions: true
			}), viewType);
			
			// leave blank so that the first section gets selected by default
			options.defaultSectionId = null;
			
			options.title= App.isRtc ?
					App.messages.SprintPlanningController_iterationPlanningLabel :
					App.messages.SprintPlanningController_sprintPlanningLabel;
			options.showLaneItemCount = true;
			
			// this is needed for setting URL location
			options.context.planType = Constants.sprintPlanning;
			
			return options;
		},
		
		createSections: function(hints, viewType) {
			var newContext = _.clone(hints.context);
			if (App.isJazzhub && newContext && !newContext.iteration) {
				var iterationId = App.controller.filterController
						.getCurrentlyShowingSprintFilterValue(
								FilterConstants.sectionAll,
								newContext.project);
				
				if (iterationId) {
					App.controller.iterationController.fetchIterationbyID(newContext.project, iterationId, function(iteration) {
						if (iteration) {
							newContext.iteration = iteration;
							newContext.timeline = iteration.get('timeline');
						}
					}, null, false);
				}
				else {
					var currentIteration = App.controller.iterationController.getDefaultSelectedIteration(newContext.project);
					if (currentIteration && currentIteration.id != FilterConstants.filterUnassigned) {
						newContext.iteration = currentIteration;
						newContext.timeline = currentIteration.get('timeline');
					}
				}
			}
			
			var iterations =this._getiterationProvider(viewType).getIterations(newContext, viewType, App);
			var sectionsArray = _.map(iterations, function(iteration) { 
				return this._createSectionForIteration(hints, iteration, viewType);
			}, this);
			
			return new SectionCollection(sectionsArray);
		},
		
		_getiterationProvider: function(viewType) {
			var iterationProviderData = {
				isMultiLane: viewType === Constants.viewTypeListLanes || viewType === Constants.viewTypeTableLanes,
				isSmallScreen: App.isSmallScreen,
				isRtc: App.isRtc,
				isJazzhub: App.isJazzhub,
				isJazzhubProject: App.isJazzhubProject,
				iterationController: App.controller.iterationController
			};
			return new IterationsProvider(iterationProviderData);
		},
		
		_createSectionForIteration: function(hints, iteration, viewType) {
			var newContext = _.clone(hints.context);
			newContext.iteration = iteration;
			
			var showProgress = this._isNotBacklogIteration(iteration, newContext.project);

			var initialExcludeResolved = App.preferences.at(0).get(FilterConstants.excludeResolvedPreferencesKey);
			if(initialExcludeResolved === undefined){
				// Default to exclude resolved work items
				initialExcludeResolved = true;
			}
			
			return new SprintPlanTreeSection({
				context: newContext,
				initialExcludeResolved: initialExcludeResolved,
				columns: this.getColumnIdentifiers(),
				sortAscending: this.getSortAscendingColumns(),
				sortDescending: this.getSortDescendingColumns(),
				getGroup: this.createGroupingFunction(iteration.get('id')),
				workItemViewOptions: {
					tableHeaders: this.getTableViewHeaders(),
					editActions: this.getEditActions(),
					emptyViewOptions: this.getEmptyViewOptions(newContext),
					isTree: true,
					decorateResolved: true,
					viewType: viewType || hints.viewType
				},
				showProgress: showProgress,
				showVelocityReports: !showProgress
			});
		},
		
		_isNotBacklogIteration: function(iteration, project){
			var backlogIteration= App.controller.iterationController.getBacklogIteration(project);
			return backlogIteration ? backlogIteration.get(CommonConstants.propId) !== iteration.get(CommonConstants.propId) : true;
		},
		
		/**
		 * Given an item and the section's functional ID, supplies the group name
		 * under which the item should fall.
		 */
		getGroup: function(item, functionalId) {
			return App.messages.BacklogController_rankedList;
		},
		
		getEmptyViewOptions: function(context) {
			var emptyViewOptions = {};
			
			if (context && context.project && App.controller.iterationController.getBacklogIteration(context.project)) {
				emptyViewOptions.message= App.messages.SprintPlanningController_noWorkItemsInSprint;
			} else {
				emptyViewOptions.message= App.messages.SprintPlanningController_noWorkItemsInSprintNoBacklog;
			}
			
			return emptyViewOptions;
		},
		
		getEditActions: function() {
			var editActions = {};
			editActions[Constants.propStatus] = true;
			editActions[Constants.propPriority] = true;
			editActions[Constants.propSeverity] = true;
			editActions[Constants.propOwnedBy] = true;
			editActions[Constants.propIsStarred] = true;
			editActions[Constants.propTags] = true;
			editActions[Constants.editRefresh] = true;
			editActions[Constants.propChildren] = true;
			editActions[Constants.propRank] = true;
			return editActions;
		},
		
		getTableViewHeaders: function() {
			var expandCollapseHeader = {name:"expand",label:"",size:"30px",align:"center"};
			var iconHeader = {name:"icon",label:"",size:"25px",align:"center",valign:"top"};
			var summaryHeader = {name:Constants.propSummary,label:"&nbsp;",align:"left",valign:"top"};
			var rankHeader = {name:Constants.propRank,label:"&nbsp;",size:"80px", align:"center"};
			var asArray = [expandCollapseHeader, iconHeader, summaryHeader, rankHeader];
			if (this.complexityAttributeID) {
				var complexityHeader= {name:Constants.propWiComplexityID, label: "", size: "60px", align:"center"};
				asArray.push(complexityHeader);
			}
			var attributeHeader = {name:"actions",label: "", size:"180px",align:"center"};
			asArray.push(attributeHeader);
			return this.formatTableViewHeaders(asArray);
		},
		
		createFilterContainerView: function(options) {
			var sprintFilterInNavbar= this.shouldDisplaySprintFilterInNavbar();
			options.groupFilterOptions= this.getGroupFilterOptions();

			if (sprintFilterInNavbar) {
				var filterContainerView= new SprintPlanningContainerView({
					context: options.context,
					groupFilterOptions: [],
					filterKey: "all",
					showSprintFilter: sprintFilterInNavbar,
					showEditSprint: this.shouldDisplayEditSprintsInNavbar(options.context)
				});
				App.navbars.setFilterSection(filterContainerView);
			} else {
				App.navbars.hideFilterSection();
			}
		}, 
		
		refreshAndValidateItem: function(workItems, attributes) {
			var updateNeeded = _.any(workItems, _.bind(function(workItem, index) { return this._parentChildModelUpdater.updateParentChildModel(workItem, attributes[index]); }, this));
			return updateNeeded ?
						AbstractWorkItemCollectionController.prototype.refreshAndValidateItem.apply(this, arguments) :
						$.Deferred().resolve(workItems);
		}
	});
});
