/**********************************************************************************************************************************
 * Licensed Materials - Property of IBM
 * Restricted Materials of IBM.  Reproduction, modification and redistribution are prohibited.
 * Task-Oriented Quick Tracking and Planning feature for Rational Team Concert and Jazzhub.
 * (c) Copyright IBM Corporation 2014.
 * U.S. Government Users Restricted Rights:  Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp. 
 **********************************************************************************************************************************/

define(['framework/App', 'backbone', 'marionette', 'uri',
	'framework/common/utils/StringUtils', 
    'areas/workitems/controllers/MyWorkController',
    'areas/workitems/controllers/InboxController',
    'areas/workitems/controllers/TeamsWorkController',
    'areas/workitems/controllers/AllWorkController',
    'areas/workitems/controllers/SubscribedToController',
    'areas/workitems/controllers/StarredController',
    'areas/workitems/controllers/RecentWorkController',
    'areas/workitems/controllers/BacklogUtil',
    'areas/workitems/controllers/BacklogPageController',
    'areas/workitems/controllers/SprintPlanningPageController',
    'areas/workitems/controllers/TaskBreakdownController',
    'areas/workitems/models/WorkItemConstants',
	'areas/workitems/attributes/models/AttributeConstants',
	'areas/servers/models/ServerConstants',
	'areas/plans/models/PlanConstants',
	'areas/workitems/models/WorkItem',
	'areas/workitems/models/WorkItemCollection',
	'areas/workitems/models/BulkWorkItemSave',
	'areas/workitems/models/WorkItemTypeCollection',
	'areas/workitems/models/WorkItemQuickCreateParser',
	'areas/workitems/models/WorkItemWorkflowCollection',
	'framework/common/models/Action',
	'areas/users/models/UserConstants',
	'areas/projects/models/ProjectConstants',
	'areas/workitems/views/QuickCreateFullscreenView',
	'areas/workitems/views/QuickEditFullscreenView',
	'framework/common/views/ViewUtil',
	'areas/common/views/RtcFrameEmbeddedView',
	'areas/activities/controllers/ActivityController'], 
function(App, Backbone, Marionette, URI, StringUtils,
	MyWorkController, InboxController, TeamsWorkController, AllWorkController, 
	SubscribedToController, StarredController, RecentWorkController, 
	BacklogUtil, BacklogPageController, SprintPlanningPageController, 
	TaskBreakdownController, Constants, AttributeConstants, ServerConstants, PlanConstants,
	WorkItem, WorkItemCollection, BulkWorkItemSave, WorkItemTypeCollection, WorkItemQuickCreateParser,
	WorkItemWorkflowCollection,
	Action, UserConstants, ProjectConstants,
	QuickCreateFullScreenView, QuickEditFullscreenView, ViewUtil, 
	RtcFrameView,
	ActivityController) {
	
	var TYPES_TO_RESOLVE_AS_INVALID_INFOS = {
		'defect' : {
			// Resolve
			actionId : "com.ibm.team.workitem.defectWorkflow.action.resolve",
			
			// Invalid
			resolutionId : "5",
			
			// Reopen
			reopenActionId : "com.ibm.team.workitem.defectWorkflow.action.reopen"
		},
		'task' : {
			// Invalidate
			actionId : "com.ibm.team.workitem.taskWorkflow.action.a2"
		},
		'com.ibm.team.apt.workItemType.story' : {
			// Reject
			actionId : "com.ibm.team.apt.storyWorkflow.action.a16"
		},
		'com.ibm.team.apt.workItemType.epic' : {
			// Reject
			actionId : "com.ibm.team.apt.epic.workflow.action.a8",
			
			reopenActionId : "com.ibm.team.apt.epic.workflow.action.a11"
		},
		'com.ibm.team.workItemType.buildtrackingitem' : {
			// Abandon
			actionId : "com.ibm.team.workitem.buildTrackingWorkflow.action.a1"
		},
		'com.ibm.team.workitem.workItemType.impediment' : {
			// Invalidate
			actionId : "com.ibm.team.workitem.impedimentWorkflow.action.a3"
		},
		'com.ibm.team.workItemType.adoption' : {
			// Reject
			actionId : "com.ibm.team.rtc.workflow.adoption.action.a5",
			
			reopenActionId : "com.ibm.team.rtc.workflow.adoption.action.a6"
		},
		'com.ibm.team.workitem.workItemType.retrospective' : {
			// Invalidate
			actionId : "com.ibm.team.workitem.retrospectiveWorkflow.action.a1"
		}
	};
	
	var MAX_RECENT_TAGS = 5;
	
	return Backbone.Marionette.Controller.extend({
		
		_isRefreshSuspended: false,
		_currentController: null,
		rtcFrameView: null,
		activityController : new ActivityController(),
		
		refresh: function(sectionId) {
			if (!this._isRefreshSuspended && this._currentController) {
				this._currentController.refresh(sectionId);
			}
		},
		
		suspendRefresh: function() {
			this._isRefreshSuspended = true;
		},
		
		resumeRefresh: function(sectionToRefreshNow) {
			this._isRefreshSuspended = false;
			if (sectionToRefreshNow) {
				this.refresh(sectionToRefreshNow);
			}
		},
		
		isRefreshSuspended: function() {
			return this._isRefreshSuspended;
		},
		
		getCurrentPaneType: function() {
			if (!this._currentController) return null;
			if (! _.isFunction(this._currentController.getCurrentEditor)) return null;
			
			var currentEditor = this._currentController.getCurrentEditor();
			if (!currentEditor) return null;
			
			return currentEditor.currentlyShowing;
		},
		
		fetchMore: function(collection) {
			collection.fetchMore();
		},
		
		showDragFeedback: function() {
			if (this._currentController) {
				this._currentController.showDragFeedback(App.itemBeingDragged);
			}
		},
		
		hideDragFeedback: function() {
			if (this._currentController) {
				this._currentController.hideDragFeedback();
			}
		},
		
		/////////////////////////////////////////////////////////////////////////////////
		// SHOW METHODS
		/////////////////////////////////////////////////////////////////////////////////
		
		/**
		 * Helper method to create and start a collection editor given the WorkItemCollectionEditor class. 
		 */
		showWorkItemCollectionEditor: function(WorkItemCollectionEditor, 
			context, viewType, paneType, dontNavigate, itemIdToShow, controllerOptions) {
			
			var startEditor= _.bind(function() {
				controllerOptions = controllerOptions || {};
				
				this.start(new WorkItemCollectionEditor(_.extend(controllerOptions, {
					context: context,
					viewType: viewType,
					paneType: paneType,
					dontNavigate: dontNavigate
				})));
			},this);
			
			var isBigScreenJazzhub= App.isJazzhub && !App.isSmallScreen;
			if (itemIdToShow) {
				startEditor();
				this.showFullscreen(this._getWorkItemUrlFromNumber(context, itemIdToShow), null, null, itemIdToShow);
			} else if (!isBigScreenJazzhub) {
				startEditor();				
			} else if (isBigScreenJazzhub && this.confirmIFrameClose()) {
				this.closeIFrameView(true);
				startEditor();
				if (paneType === Constants.viewTypeListLanes ||
						paneType === Constants.viewTypeTableLanes) {
					$('body').css('overflow', 'hidden');
				} else {
					$('body').css('overflow', 'visible');
				}
			}
		},
		
		/**
		 * Helper method to start a collection editor. 
		 */
		start: function(workItemController) {
			this._currentController = workItemController;
			workItemController.start();
		},
		
		/*
		 * Displays the My Work Item view in the editor, and selects
		 * the default 
		 */
		showInbox: function(context, viewType, dontNavigate, itemIdToShow) {			
			this.showWorkItemCollectionEditor(InboxController, context, 
					viewType, Constants.paneTypeInbox, dontNavigate, itemIdToShow);
			App.controller.planController.broadcastViewChanged(Constants.inbox);
		},
		
		showMyWork: function(context, viewType, paneType, dontNavigate, itemIdToShow) {
			context = _.clone(context);
			context.newWorkItemOwnerIsCurrentUser = true;
			
			this.showWorkItemCollectionEditor(MyWorkController, context, 
					viewType, paneType, dontNavigate, itemIdToShow);
			App.controller.planController.broadcastViewChanged(Constants.planTypeMyWork);
		},
		
		/*
		 * Displays the My Work Item view in the editor, and selects
		 * the default 
		 */
		showTeamsWork: function(context, viewType, paneType, dontNavigate, itemIdToShow) {
			this.showWorkItemCollectionEditor(TeamsWorkController, context, 
					viewType, paneType, dontNavigate, itemIdToShow);
			App.controller.planController.broadcastViewChanged(Constants.teamWork);
		},
		
		showAllWork: function(context, viewType, paneType, dontNavigate, itemIdToShow) {
			this.showWorkItemCollectionEditor(AllWorkController, context, 
					viewType, paneType, dontNavigate, itemIdToShow);
			App.controller.planController.broadcastViewChanged(Constants.allWork);
		},
		
		showSubscribedTo: function(context, viewType, paneType, dontNavigate, itemIdToShow) {
			this.showWorkItemCollectionEditor(SubscribedToController, context, 
					viewType, paneType, dontNavigate, itemIdToShow);
			App.controller.planController.broadcastViewChanged(Constants.subscribedTo);
		},
		
		showStarred: function(context, viewType, paneType, dontNavigate, itemIdToShow) {
			this.showWorkItemCollectionEditor(StarredController, context, 
					viewType, paneType, dontNavigate, itemIdToShow);
			App.controller.planController.broadcastViewChanged(Constants.starred);
		},
		
		showRecentWork: function(context, viewType, paneType, dontNavigate, itemIdToShow) {
			this.showWorkItemCollectionEditor(RecentWorkController, context, 
					viewType, paneType, dontNavigate, itemIdToShow);
			App.controller.planController.broadcastViewChanged(Constants.recentWork);
		},
		
		showMyActivities: function(context, ActivityLayoutView, ActivityCollectionView, itemIdToShow) {
			context.planType = Constants.myActivities;
			context.area = "items";
			this.activityController.showForJazzhub(ActivityLayoutView, 
					ActivityCollectionView, context, App.extras, itemIdToShow);
			App.controller.planController.broadcastViewChanged(Constants.myActivities);
		},
		
		showBacklog: function(context, viewType, paneType, dontNavigate, itemIdToShow) {
			this.showWorkItemCollectionEditor(BacklogPageController, context, 
				viewType, paneType, dontNavigate, itemIdToShow, {isBacklogOnlyView: true});
			App.controller.planController.broadcastViewChanged(Constants.backlog);
		},
		
		showSprintPlanning: function(context, viewType, paneType, dontNavigate, itemIdToShow) {	
			this.showWorkItemCollectionEditor(SprintPlanningPageController, context, 
					viewType, paneType, dontNavigate, itemIdToShow);
			App.controller.planController.broadcastViewChanged(Constants.sprintPlanning);
		},
		
		showTaskBreakdown: function(context, viewType, dontNavigate, parentWorkItemId, parentWorkItem, itemIdToShow) {
			context = _.extend(context, {
				parentWorkItemId: parentWorkItemId,
				parentWorkItem: parentWorkItem
			});
			this.showWorkItemCollectionEditor(TaskBreakdownController, context, 
				viewType, null, dontNavigate, itemIdToShow, {
					previousController: this._currentController
				}
			);
		},
		
		showParentTaskBreakdown: function(context, viewType, dontNavigate, childWorkItem) {
			// we fetch the child work item's parent, then fetch its parent, then give it to the task breakdown view
			var getId = function(workItem) {
				if (workItem) {
					var id;
					if (_.isFunction(workItem.getValue)) {
						id = workItem.getValue('id').id;
					} else {
						id = workItem.get(Constants.propNumber);
					}
					
					return id;
				}
				
				return null;
			};
			
			var self = this;
			
			var fetchParentWorkItem = function(workItemId) {
				var parentItemCollection = new WorkItemCollection([], context);
				parentItemCollection.fetchWorkItemByNumber(workItemId, {
					success: function() {
						if (parentItemCollection.size() === 1) {
							var parentItem = parentItemCollection.first();
							var parentId = getId(parentItem);
							var newContext = _.clone(context);
							newContext.collection = parentItemCollection;
							self.showTaskBreakdown(context, viewType, dontNavigate, parentId, parentItem);
						}
					}
				}, context, [
					Constants.propId,
					Constants.propType,
					Constants.propParent,
					Constants.propPlannedFor,
					Constants.propSummary
				]);
			};
			
			var parentProxy = childWorkItem.get(Constants.propParent);
			if (parentProxy) {
				fetchParentWorkItem(getId(parentProxy));
			} else {
				// if the parent property is not there, we need to fetch this item first...
				var getItemCollection = new WorkItemCollection([], context);
				var id = getId(childWorkItem);
				
				getItemCollection.fetchWorkItemByNumber(id, {
					success: function() {
						if (getItemCollection.size() === 1) {
							// now fetch the parent
							parentProxy = getItemCollection.first().get(Constants.propParent);
							fetchParentWorkItem(getId(parentProxy));
						}
					}
				}, context, [Constants.propParent]);
			}
		},
		
		_getWorkItemUrlFromNumber: function(context, workItemNumber) {
			var url = new URI(context.project.get('url'));
			url.hash('action=com.ibm.team.workitem.viewWorkItem&id=' + workItemNumber);
			return url.toString();
		},
		
		ensureIFrameInitialized: function(project) {
			if (this.rtcFrameView || !project || !App.isJazzhub || App.isSmallScreen) {
				return;
			}
			
			var webUrl = project.get(ServerConstants.propUrl);
			webUrl = new URI(webUrl);
			webUrl.hash('action=com.ibm.team.workitem.viewWelcome');
			var url = webUrl.toString();
			
			this.rtcFrameView = new RtcFrameView();
			this.rtcFrameView.openInStealthMode(url);
		},
		
		showInIFrameView: function(workItem, successHandler, anotherItemToRefresh, itemId) {
			var notaWorkitem = _.isString(workItem);
			var number = itemId, url;
			
			if (notaWorkitem) {
				if (!number) {
					number = new URI(workItem).segment(-1);
				}
				url = workItem;
			} else {
				number = workItem.get(Constants.propNumber);
				
				var project = workItem.get(Constants.propProject);
				var webUrl = project.get(ServerConstants.propUrl);
				webUrl = new URI(webUrl);
				webUrl.hash('action=com.ibm.team.workitem.viewWorkItem&id=' + workItem.get(Constants.propNumber));
				url = webUrl.toString();
			}
			
			this.rtcFrameView.openWorkItem(url, number, _.bind(function() {
				// success
				if (successHandler) {
					successHandler();
				} else if (notaWorkitem && anotherItemToRefresh) {
					this.refreshAndValidateItem(anotherItemToRefresh);
				} else if (!notaWorkitem) {
					this.refreshAndValidateItem(workItem);
				}
			}, this));
		},
		
		closeIFrameView: function(doRefresh) {
			if(this.rtcFrameView) {
				this.rtcFrameView.closeWorkItem(doRefresh);
			}
		},
		
		confirmIFrameClose: function() {
			var confirmed= true;
			if(this.rtcFrameView && App.controller.getWorkItemNumberBeingEdited()) {
				confirmed= this.rtcFrameView.confirmClose();
			}
			return confirmed;
		},
		
		discardIFrameChanges: function() {
			if(this.rtcFrameView) {
				this.rtcFrameView.discardIFrameChanges();
			}
		},
		
		/**
		 * If workItem is a string, then it will be treated as a URI and shown in full screen.
		 * However, anotherItemToRefresh will be refreshed in the UI instead.
		 */
		showFullscreen: function(workItem, successHandler, anotherItemToRefresh, itemId) {
			var notaWorkitem = _.isString(workItem);
			
			var uri;
			if (notaWorkitem) {
				uri = workItem;
			} else {
				/*
				 * Transform the URI to go directly to the web UI
				 */
				var project = workItem.get(Constants.propProject);
				var webUrl;
				if (App.isRtc && App.isEmbeddedInRtc) {
					// To preserve ?debug=true for developers
				    webUrl = window.parent.location.href.replace(/#.*/, "");
				} else {
				    webUrl = project.get(ServerConstants.propUrl);
				}
				webUrl = new URI(webUrl);
				webUrl.hash('action=com.ibm.team.workitem.viewWorkItem&id=' + workItem.get(Constants.propNumber));
				uri = webUrl.toString();
			}
			
			if (App.isJazzhub) {
				if (!App.isSmallScreen) {
					this.showInIFrameView(workItem, successHandler, anotherItemToRefresh, itemId);
				} else {
					/*
					 * Note: there is currently no clean way to show the 
					 * user more information about a work item on small screens.
					 * This is mainly because the work item editor is not responsive
					 * and does not fit well in our responsive UI.
					 * 
					 * Hence, we are going to redirect the user to the Work Item.
					 */
					window.open(uri, '_blank');
				}
			} else if (App.isRtc) {
			    App.navigateToUrl(uri);
			} else {
				// mobile...
				
				/*
				 * TODO need to test this on mobile. Old Cordova doc said _system, but
				 * newer says _blank will work. 
				 */
				window.open(uri, '_blank', 'location=yes');
			}
			
			if (notaWorkitem && anotherItemToRefresh) {
				this.addToRecentWorkItems(anotherItemToRefresh);
			} else if (!notaWorkitem) {
				this.addToRecentWorkItems(workItem);
			}
		},
		
		setTitle: function(editorTitle, contextStr) {
			if (editorTitle && contextStr) {
				App.layout.setTitle(this._currentController.getTitleText(editorTitle, contextStr), true);
			} else {
				var currentEditor = this._currentController.getCurrentEditor();
				if (currentEditor) {
					currentEditor.setTitle();
				}
			}
		},
		
		/////////////////////////////////////////////////////////////////////////////////
		// EDITING METHODS
		/////////////////////////////////////////////////////////////////////////////////
		
		createWorkItem: function(context, quickCreateText, attributeCollection, monitor, successCallback, errorCallback, options) {
			var initialStateGroupId = options ? options.stateGroupId : null;
			
			if (context) {
				if (monitor) {
					monitor.trigger("create");
				}
				
				var parser = this.newQuickCreateParser(quickCreateText, this._currentController ? this._currentController.customizeContextForQuickCreate(_.clone(context)) : context);
				
				parser.parse(_.bind(function(attributes) {
					var rawAttributes = {};
					_.forIn(attributes, _.bind(function(attribute, key) {
						rawAttributes[key] = App.controller.workItemAttributeController.getRawAttributeValue(key, attribute.values, context);
						
						if (Constants.propTags === key) {
							this.addToRecentTags(attribute.values.join(', '), context.project);
						}
					}, this));
					if (attributeCollection) {
						attributeCollection.forEach(function(attribute) {
							var id = attribute.get(AttributeConstants.propId);
							// On large screen, we don't want the attributes in the collection to override what's defined in the text
							// On small screens, this is allowed since the UI is the main way to set attributes
							if (!rawAttributes[id] || App.isSmallScreen) {
								if (attribute.get(AttributeConstants.propValue)) {
									rawAttributes[id] = attribute.get(AttributeConstants.propValue);
								}
							} 
						});
					}
					
					if (initialStateGroupId) {
						// Support to create work items in an initial state group, (eg. In Progress), if possible
						// given the project's workflow. The services we use to create work items do not support
						// an initial state, so we make a second request
						var origSuccessCallback = successCallback;
						successCallback = _.bind(function(newWorkItem) {
							if (newWorkItem) {
								WorkItemWorkflowCollection.withWorkItemWorkflow(context.project, newWorkItem, _.bind(function(workflow) {
									var actions = workflow.getActions(newWorkItem.get(Constants.propStatus), initialStateGroupId);
									if (actions.length === 0) { 
										origSuccessCallback();
									} else {
										var attrs = {};
										attrs[Constants.propStatus] = actions[0].get(Constants.propId);
										this.updateWorkItem(context, newWorkItem, attrs, monitor, origSuccessCallback, errorCallback);
									}
								}, this));
							} else {
								origSuccessCallback();
							}
						}, this);
					}
					
					var options = this._getPostWorkItemOptions(context, null, rawAttributes, true, monitor, successCallback, errorCallback, App.messages.WorkItemController_createError);
					var workItem = new context.collection.model(Constants.reduceToContextFieldsOnly(context));
					context.collection.create(workItem, options);
				}, this), options);
			}
		},
		
		getRecentTags: function(project) {
			var key = project.id + ":" + Constants.recentlySelectedTagsPreferencesKey,
				preferences = App.preferences.at(0);
			return preferences.get(key) || [];
		},
		
		addToRecentTags: function(tagsStr, project) {
			var key = project.id + ":" + Constants.recentlySelectedTagsPreferencesKey,
				preferences = App.preferences.at(0),
				recentlySelectedTags = preferences.get(key) || [],
				allTags = tagsStr.split(/[, ]/);
				
			_.forEach(allTags, function(tag) {
				tag = tag.trim();
				if (tag.length !== 0) {
					var length = recentlySelectedTags.unshift(tag);
					if (length > MAX_RECENT_TAGS) {
						recentlySelectedTags = recentlySelectedTags.slice(0, MAX_RECENT_TAGS);
					}
				}
			});
			
			preferences.set(key, recentlySelectedTags);
			preferences.save();
		},
		
		clearRecentTags: function(project) {
			var key = project.id + ":" + Constants.recentlySelectedTagsPreferencesKey,
				preferences = App.preferences.at(0);
				
			preferences.unset(key);
			preferences.save();
		},
		
		_getPostWorkItemOptions: function(context, workItem, attributes, parametrize, monitor, successCallback, errorCallback, errorSummary) {
			var query = {};
			query[Constants.attributeIdentifiers] = [];
			query[Constants.attributeValues] = [];
			query[Constants.updateLinks] = [];
			if (workItem) {
				if (workItem.get(Constants.propItemId)) {
					query[Constants.propItemId] = workItem.get(Constants.propItemId);
				} else {
					query[Constants.propItemId] = workItem.get(Constants.propWorkItemItemId);					
				}
				query[Constants.propStateId] = workItem.get(Constants.propStateId);
			} else {
				query.projectAreaItemId= context.project.get(Constants.propItemId);
			}
			
			var parentToRemove = null;
			var newParentAdded = null;
			var refreshForParentOnly = false;
			
			_.forIn(attributes, function(value, key) {
				if (Constants.propStatus === key) {
					//Status is not updated directly; in this case the value is an action
					//identifier
					query[Constants.action] = value;
				} else if (Constants.propParent === key) {
					if (workItem) {
						var currentParent = workItem.get(Constants.propParent);
						if (currentParent && App.messages.WorkItemController_unassigned !== currentParent) {
							parentToRemove = currentParent;
							
							// the item can be a proxy or a complete work item.
							var itemId = currentParent.get(Constants.propWorkItemItemId);
							if (!itemId) {
								itemId = currentParent.get(Constants.propItemId);
							}
							var url = currentParent.get(Constants.propUrl);
							if (!url) {
								url = currentParent.get(Constants.propId);
							}
							query[Constants.updateLinks].push(JSON.stringify({
								cmd: "removeLink",
								type: "com.ibm.team.workitem.linktype.parentworkitem",
								end: "target",
								name: App.messages.WorkItemController_parent,
								itemId: itemId,
								uri: url
							}));
						}
					}
					
					if (value !== null) {
						//For the attribute controller and views, it's easier to think of
						//work item attributes as enumeration values. Here, we need the
						//actual work item model.
						value = value.get("workItemModel") || value;
						newParentAdded = value;
						query[Constants.updateLinks].push(JSON.stringify({
							cmd: "addLink",
							type: "com.ibm.team.workitem.linktype.parentworkitem",
							end: "target",
							name: App.messages.WorkItemController_parent,
							itemId: value.get(Constants.propWorkItemItemId),
							comment: "{0}: {1}".format(value.get(Constants.propNumber), value.get(Constants.propSummary))
						}));
					}
					
					refreshForParentOnly = true;
				} else if (Constants.propChildren === key) {					
					if (value !== null) {
						//For the attribute controller and views, it's easier to think of
						//work item attributes as enumeration values. Here, we need the
						//actual work item model.
						value = value.get("workItemModel") || value;
						query[Constants.updateLinks].push(JSON.stringify({
							cmd: "addLink",
							type: "com.ibm.team.workitem.linktype.parentworkitem",
							end: "source",
							name: App.messages.WorkItemController_children,
							itemId: value.get(Constants.propWorkItemItemId),
							comment: "{0}: {1}".format(value.get(Constants.propNumber), value.get(Constants.propSummary))
						}));
						
						refreshForParentOnly = true;
					}
				} else if (Constants.propDuplicateOf === key) {					
					if (value !== null) {
						value = value.get("workItemModel") || value;
						query[Constants.updateLinks].push(JSON.stringify({
							cmd: "addLink",
							type: "com.ibm.team.workitem.linktype.duplicateworkitem",
							end: "source",
							name: App.messages.WorkItemController_duplicateOf,
							itemId: value.get(Constants.propWorkItemItemId),
							comment: "{0}: {1}".format(value.get(Constants.propNumber), value.get(Constants.propSummary))
						}));
					}
				} else {
					query[Constants.attributeIdentifiers].push(key);
					query[Constants.attributeValues].push(value);
				}
			});
			
			var url = context.server.get(ServerConstants.propUrl);
			url = new URI(url);
			url.segment(ServerConstants.service)
				.segment("com.ibm.team.workitem.common.internal.rest.IWorkItemRestService")
				.segment("workItem2");
			var collection = workItem ? workItem.collection : context.collection;
			return {
				url: url.toString(),
				type: "POST",
				data: parametrize ? $.param(query, true) : query, // true is for traditional parameterization for arrays
				reset: true,
				refreshForParentOnly: refreshForParentOnly,
				success: _.bind(function(newWorkItem, updatedAttributes) {
					if (newWorkItem.get(Constants.propError)[Constants.propErrorSeverity]) {
						if (monitor) {
							monitor.trigger("error");
						}
						
						if (errorCallback) {
							errorCallback(
								newWorkItem.get(Constants.propError)[Constants.propErrorSeverity],
								newWorkItem.get(Constants.propError)[Constants.propErrorSummary],
								newWorkItem.get(Constants.propError)[Constants.propErrorDescription]
							);
						}
					} else {
						if (monitor) {
							monitor.trigger("sync");
						}
					
						if (successCallback) {
							successCallback(newWorkItem, updatedAttributes);
						}
						this.addToRecentWorkItems(newWorkItem);
						if(collection){
							collection.trigger("workItemSaved");
						}
					}
				}, this),
				error: function(model, response, options) {
					if (monitor) {
						monitor.trigger("error", model, response, options);
					}
					if (errorCallback) {
						var description = response.responseJSON ? response.responseJSON.errorMessage : "";

						var errorObj= {};
						errorObj[Constants.propErrorSeverity]= 4;
						errorObj[Constants.propErrorSummary]= errorSummary;
						errorObj[Constants.propErrorDescription]= description;

						model.set(Constants.propError, errorObj);
						errorCallback(4, errorSummary, description);
					}
				},
				dataType: Constants.dataTypes_json
			};
		},
		
		newQuickCreateParser: function(quickCreateText, context) {
			return new WorkItemQuickCreateParser(quickCreateText, context);
		},
		
		/**
		 * In smallscreen, displays a full-screen UI to "quick create" a work item in the
		 * full-screen sidebar of the application.
		 * 
		 * Othwerwise, toggles the display of the same UI in the work item collection view's
		 * getExpandedQuickCreateRegion().
		 */
		showExpandedQuickCreate: function(context, expandedQuickCreateRegion, alertPlaceholderEl, monitor, textElement, options) {
			if (App.isSmallScreen) {
				var quickCreateView = new QuickCreateFullScreenView({
					context: context,
					initialText: textElement.prop("value")
				});
				
				App.sidebars.openFullScreenRight(quickCreateView);
				
				var attributeCollection = App.controller.workItemAttributeController.showEditAttributesForQuickCreate(quickCreateView.attributesSection, quickCreateView.getAlertPlaceholderEl(), textElement, context, monitor);
				
				App.layout.setTitle(App.messages.WorkItemController_addTask, true);
				
				var goBack = _.bind(function() {
					App.sidebars.closeFullScreenRight();
				}, this);
				
				var cancelAction = new Action({
					icon: 'icon-arrow-back',
					handler: function() {
						monitor.trigger("cancel");
						goBack();
					}
				});
					
				App.layout.showNavigationActions(cancelAction, true);
				
				var saveAction = new Action({
					icon: 'glyphicon glyphicon-ok',
					handler: _.bind(function() {
						this.createWorkItem(context, quickCreateView.getText(), attributeCollection, monitor,
							//success
							goBack,
							
							//failure
							_.bind(function(severity, summary, description) {
								App.showAlert(quickCreateView.getAlertPlaceholderEl(), {
									summary: summary,
									description: description
								});
							}, this),
							
							options
						);
					}, this)
				});
				
				App.layout.showEditActions(saveAction, true);
				
				//Do not return the attributes, since we only want the local variable
				//attributeCollection to be used in the success handler for createWorkItem.
				return undefined;
			} else {
				if (expandedQuickCreateRegion.currentView) {
					expandedQuickCreateRegion.close();
				} else {
					return App.controller.workItemAttributeController.showEditAttributesForQuickCreate(expandedQuickCreateRegion, alertPlaceholderEl, textElement, context, monitor);
				}
			}
		},
		
		hideExpandedQuickCreate: function(expandedQuickCreateRegion, textElement) {
			if (!App.isSmallScreen) {
				expandedQuickCreateRegion.close();
			}
			App.controller.workItemAttributeController.removeKeyListener(textElement);
		},
		
		/**
		 * In smallscreen, displays a full-screen UI to "edit" a work item in the
		 * full-screen sidebar of the application.
		 * 
		 */
		openFullscreenEdit: function(context, monitor, model) {
			var quickEditView = new QuickEditFullscreenView({
				context: context,
				model: model
			});
				
			App.sidebars.openFullScreenRight(quickEditView);
				
			var attributeCollection = App.controller.workItemAttributeController.editAttributesFullScreen(quickEditView.attributesSection, quickEditView.getAlertPlaceholderEl(), model, context, monitor);
			
			App.layout.setTitle(App.messages.WorkItemController_title + model.get('number'), true);
				
			var goBack = _.bind(function() {
				App.sidebars.closeFullScreenRight();
			}, this);
				
			var cancelAction = new Action({
				icon: 'icon-arrow-back',
				handler: function() {
					monitor.trigger("cancel");
					goBack();
				}
			});
					
			App.layout.showNavigationActions(cancelAction, true);
				
			var saveAction = new Action({
				icon: 'glyphicon glyphicon-ok',
				handler: _.bind(function() {
					var attributesUpdated = this.findAttributesUpdated(model, attributeCollection.models, quickEditView.getText());
					if (_.size(attributesUpdated) > 0) {
						this.updateWorkItem(context, model, attributesUpdated, monitor, 
								//success
								goBack,
								//failure
								_.bind(function(severity, summary, description) {
									App.showAlert(quickEditView.getAlertPlaceholderEl(), {
									summary: summary,
									description: description
									});
								}, this)
						);
					}
				}, this)
			});
				
			App.layout.showEditActions(saveAction, true);
		},

		findAttributesUpdated: function(wiModel, attributeCollection, newSummaryText){
			//Iterate thru the attributeCollection and for each attribute, compare the displayValue
			//with the value present in the workItem model, only where it is different
			//add them to the attributes object with the key as id and value as the displayValue
			var attributeUpdated = {};
			
			_.each(attributeCollection, function(attribute) {
				if (attribute.changedAttributes()){
					attributeUpdated[attribute.get(Constants.propId)] = attribute.get(AttributeConstants.propValue);
					
					var secondaryAttributes = attribute.get(AttributeConstants.propSecondaryAttributes);
					if (secondaryAttributes) {
						_.forEach(secondaryAttributes, function(secondaryAttribute) {
								if (secondaryAttribute.has(AttributeConstants.propValue)) {
									attributeUpdated[secondaryAttribute.get(Constants.propId)] = secondaryAttribute.get(AttributeConstants.propValue);
								}
						});
					}

				}
			},this);
			
			//Summary field is outside the attributeCollection and needs separate handling.
			if (wiModel.get(Constants.propSummary) !== newSummaryText){
				attributeUpdated.summary = newSummaryText;
			}

			return attributeUpdated;
		},
		
		refreshAndValidateItem: function(workItem, successFn, errorFn, attributes) {
			successFn= successFn || _.noop;
			errorFn= errorFn || _.noop;

			var wrapped= {
					successFn: _.isArray(workItem) ? successFn : function(workItems) { successFn(workItems[0], attributes); },
					workItems: _.isArray(workItem) ? workItem : [ workItem ],
					attributes: _.isArray(attributes) ? attributes : [ attributes ]
			};

			return this._currentController.refreshAndValidateItem(wrapped.workItems, wrapped.attributes).done(wrapped.successFn).fail(errorFn);
		},
		
		updateWorkItem: function(context, workItem, attributes, monitor, successCallback, errorCallback) {
			workItem.clearErrorAttributes();
			
			if (context) {
				if (monitor) {
					monitor.trigger("update", {
						attributes: attributes
					});
				}
				
				var errorText = App.messages.WorkItemController_updateError;
					
				//Call fetchAttributes() to populate the work item's itemId and stateId if they
				//are not already loaded
				workItem.fetchAttributes([],
					_.bind(function(stateId) {
						var options = this._getPostWorkItemOptions(context, workItem, attributes, true, monitor, successCallback, errorCallback, errorText);
						var innerSuccessCallback = options.success;
						var innerErrorCallback = options.success;
						options.success = _.bind(function() {
							if (workItem.get(Constants.propError)[Constants.propErrorSeverity]) {
								// Skip the refresh if there was a problem saving and let the caller handle
								innerSuccessCallback(workItem);
							} else {
								this.refreshAndValidateItem(workItem, innerSuccessCallback, innerErrorCallback, attributes);
								
								// also mark the work item as pending refresh in the work item editor.
								if (this.rtcFrameView && this.rtcFrameView.pendingRefresh) {
									this.rtcFrameView.pendingRefresh.push(workItem.id);
								}
							}
						}, this);
						workItem.update(options);	
					}, this),
					
					function(response) {
						if (monitor) {
							monitor.trigger("error");
						}
						if (errorCallback) {
							errorCallback(null, errorText, response.statusText);
						}
					}
				);
			}
		},
		
		updateWorkItems: function(context, workItems, attributes) {
			_.each(workItems, function(workItem) { workItem.clearErrorAttributes(); });

			var self= this,
				result= $.Deferred(),
				errorText = App.messages.WorkItemController_updateError,
				fetched= $.when(_.map(workItems, function(workItem) {
					var result= $.Deferred();
					workItem.fetchAttributes([], result.resolve, result.reject);
					return result;
				}));

			fetched
			.done(function() {
				var saveOptions= _.map(workItems, function(workItem, index) { return self._getPostWorkItemOptions(context, workItem, attributes[index], false, null, _.noop, _.noop, errorText); });
				
				BulkWorkItemSave
				.execute(workItems, saveOptions)
				.done(function(workItems) {
					self.refreshAndValidateItem(workItems, _.noop, _.noop, attributes).done(result.resolve).fail(result.reject);
					
					// also mark the work item as pending refresh in the work item editor.
					if (self.rtcFrameView && self.rtcFrameView.pendingRefresh) {
						_.each(workItems, function(workItem) { self.rtcFrameView.pendingRefresh.push(workItem.id); });
					}
				})
				.fail(result.reject);
			})
			.fail(result.reject);
			
			return result.promise();
		},
		
		moveWorkItemToBacklog: function(context, workItem, monitor, successCallback, errorCallback) {
			var collection = workItem.collection;
			
			var workItemSummary = workItem.get(Constants.propSummary);
			var workItemNumber = workItem.get(Constants.propNumber);
			var project = workItem.get(Constants.propProject);
			
			var backlogIterations = BacklogUtil.searchForBacklogIteration(project);
			
			var success = _.bind(function() {
				collection.trigger("workItemTriaged", workItem, {
					previousIteration: "",
					toBacklog: backlogIterations.length === 1,
					context: context
				});
				if (successCallback) {
					successCallback();
				}
			}, this);
			
			if (backlogIterations.length === 1) {
				var attributes = _.object([[Constants.propPlannedFor, backlogIterations[0].get(Constants.propId)]]);
				this.updateWorkItem(context, workItem, attributes, monitor, success, errorCallback);
			} else {
				var message, alertClass;
				if (backlogIterations.length === 0) {
					message = App.messages.WorkItemController_noBacklogIteration.format(workItemSummary, workItemNumber);
					alertClass = "alert-warning";
				} else {
					message = App.messages.WorkItemController_selectAnIteration.format(workItemSummary, workItemNumber);
					alertClass = "alert-info";
				}
				App.controller.workItemAttributeController.editAttribute(workItem, Constants.propPlannedFor, monitor, success, errorCallback, {
					editorArgs: {
						message: {
							message: message,
							alertClass: alertClass
						},
						filterValues: _.object([["", true]]),
						noCurrentValue: true
					}
				});
			}
			
		},
		
		moveWorkItemToTrash: function(context, workItem, monitor, successCallback, errorCallback) {
			var typeLabel = workItem.get(Constants.propType);
			var collection = workItem.collection;
			if (monitor) {
				monitor.trigger("update");
			}
			WorkItemTypeCollection.withWorkItemTypeCollection(context.project, _.bind(function(types) {
				var typeModel = _.find(types.models, function(type) {
					return type.get(Constants.propLabel) === typeLabel;
				}, this);
				var typeId = typeModel.get(Constants.propId);
				var resolutionInfo = TYPES_TO_RESOLVE_AS_INVALID_INFOS[typeId];
				if (resolutionInfo) {
					context.server.fetchUnassignedUser().done(_.bind(function(unassignedUser) {
						var attributes = {};
						attributes[Constants.propStatus] = resolutionInfo.actionId;
						if (resolutionInfo.resolutionId !== undefined) {
							attributes[Constants.propResolution] = resolutionInfo.resolutionId;
						}
						attributes[Constants.propOwnedBy] = unassignedUser.get(Constants.propId);
						var success = _.bind(function() {
							collection.trigger("workItemTriaged", workItem, {
								reopenActionId: resolutionInfo.reopenActionId || null,
								context: context
							});
							if (successCallback) {
								successCallback();
							}
						}, this);
						this.updateWorkItem(context, workItem, attributes, monitor, success, errorCallback);
					}, this));
				} else {
					console.warn("No resolution info for type " + typeId);
					if (monitor) {
						monitor.trigger("sync");
					}
					successCallback();
				}
			}, this));
		},
		
		canReopenWorkItem: function(workItem) {
			var canReopen = false;
			var workItemTypes = workItem.get(Constants.propProject).get(ProjectConstants.propWiTypes);
			var typeLabel = workItem.get(Constants.propType);
			if (workItemTypes) {
				workItemTypes.each(function(type) {
					if (type.get(Constants.propLabel) === typeLabel) {
						var resolutionInfo = TYPES_TO_RESOLVE_AS_INVALID_INFOS[type.get(Constants.propId)];
						canReopen = resolutionInfo && resolutionInfo.reopenActionId !== undefined;
						return false;
					}
				});
			}
			
			return canReopen;
		},
		
		moveWorkItemToInbox: function(context, workItem, monitor, successCallback, errorCallback) {
			var typeLabel = workItem.get(Constants.propType);
			var collection = workItem.collection;
			if (monitor) {
				monitor.trigger("update");
			}
			WorkItemTypeCollection.withWorkItemTypeCollection(context.project, _.bind(function(types) {
				var typeModel = _.find(types.models, function(type) {
					return type.get(Constants.propLabel) === typeLabel;
				}, this);
				var typeId = typeModel.get(Constants.propId);
				var resolutionInfo = TYPES_TO_RESOLVE_AS_INVALID_INFOS[typeId];
				if (resolutionInfo && resolutionInfo.reopenActionId) {
					var attributes = {};
					attributes[Constants.propStatus] = resolutionInfo.reopenActionId;
					attributes[Constants.propResolution] = "";
					var success = _.bind(function() {
						if (successCallback) {
							successCallback();
						}
					}, this);
					this.updateWorkItem(context, workItem, attributes, monitor, success, errorCallback);
				} else {
					console.warn("No reopenActionId info for type " + typeId);
					if (monitor) {
						monitor.trigger("sync");
					}
					successCallback();
				}
			}, this));
		},

		/////////////////////////////////////////////////////////////////////////////////
		// MISC/UTILITY METHODS
		/////////////////////////////////////////////////////////////////////////////////
		
		addToRecentWorkItems: function(workItem) {
			var newItemId;
			var projectId;
			if (_.isString(workItem)) {
				newItemId = workItem;
				// use the project we are currently connected to
				var params = new URI(window.location.href).hash();
				if (params) {
					var colonIndexOf = params.indexOf(":");
					if (colonIndexOf > 0) {
						params = params.substring(colonIndexOf+1, params.length);
						var deparams = $.deparam(params);
						if (deparams) {
							projectId = deparams.projectId;
						}
					}
				}
			} else {
				newItemId = workItem.get(Constants.propNumber);
				projectId = workItem.get(Constants.propProject).id;
			}
			
			// couldn't locate a project, won't find a key
			if (!projectId) {
				return;
			}
			
			var key = Constants.recentWorkItemsPreferencesKey + projectId;
			var preferences = App.preferences.at(0);
			var recentItems = preferences.get(key) || [];
			
			// Remove the item from the list, in order to move it to the front
			recentItems = _.filter(recentItems, function(itemId) {
				return itemId !== newItemId;
			});
			
			// Add the item to the front of the list
			recentItems.unshift(newItemId);
			
			// If the list is longer than the maxium, remove items 
			while (recentItems.length > Constants.MAX_RECENT_WORK_ITEMS) {
				recentItems.pop();
			}
			
			preferences.set(key, recentItems);
			preferences.save();
		},
		
		toggleIsStarred: function(workItem) {
			var isStarred = workItem.get(Constants.propIsStarred) === true;
			if (isStarred) {
				this.removeFromStarredItems(workItem);
			} else {
				this.addToStarredItems(workItem);
			}
			if (this._currentController) {
				this._currentController.isStarredChanged(workItem, !isStarred);
			}
		},
		
		addToStarredItems: function(workItem) {
			var key = Constants.starredWorkItemsPreferencesKey + workItem.get(Constants.propProject).id;
			var preferences = App.preferences.at(0);
			var starredItems = preferences.get(key) || [];
			var workItemId = workItem.get(Constants.propNumber);
			
			starredItems.push(workItemId);
			
			preferences.set(key, starredItems);
			preferences.save();
			
			workItem.clearErrorAttributes();
			workItem.set(Constants.propIsStarred, true);
		},
		
		removeFromStarredItems: function(workItem) {
			var key = Constants.starredWorkItemsPreferencesKey + workItem.get(Constants.propProject).id;
			var preferences = App.preferences.at(0);
			var starredItems = preferences.get(key) || [];
			var workItemId = workItem.get(Constants.propNumber);
			
			starredItems = _.filter(starredItems, function(itemId) {
				return itemId !== workItemId;
			});
			
			preferences.set(key, starredItems);
			preferences.save();
			
			workItem.clearErrorAttributes();
			workItem.set(Constants.propIsStarred, false);
		},
		
		fetchStarredItemIds: function(project, successCallback) {
			var key = Constants.starredWorkItemsPreferencesKey + project.id;
			var preferences = App.preferences.at(0);
			var starredItems = preferences.get(key) || [];
			var starredItemsMap = {};
			_.forEach(starredItems, function(workItemId) {
				starredItemsMap[workItemId] = true;
			});
			
			//Eventually, this may require a server call, (23567). The setTimeout() is to
			//ensure that the flow is the same for clients of this function if/when this
			//change is made
			setTimeout(function() {
				successCallback(starredItemsMap);
			});
		}, 
		
        createParentChildLink: function(newParent, newChild, monitor, successCallback, errorCallback){
			this.updateWorkItem(newParent.collection.context, newChild, _.object([[Constants.propParent, newParent]]), monitor, successCallback, errorCallback);
		}
	});
});
